/*
 * Copyright (c) 2002-2011 LWJGL Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'LWJGL' nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lwjgl.opengles;

public interface NV_fbo_color_attachments {

	int GL_MAX_COLOR_ATTACHMENTS_NV = 0x8CDF,
		GL_COLOR_ATTACHMENT0_NV     = 0x8CE0,
		GL_COLOR_ATTACHMENT1_NV     = 0x8CE1,
		GL_COLOR_ATTACHMENT2_NV     = 0x8CE2,
		GL_COLOR_ATTACHMENT3_NV     = 0x8CE3,
		GL_COLOR_ATTACHMENT4_NV     = 0x8CE4,
		GL_COLOR_ATTACHMENT5_NV     = 0x8CE5,
		GL_COLOR_ATTACHMENT6_NV     = 0x8CE6,
		GL_COLOR_ATTACHMENT7_NV     = 0x8CE7,
		GL_COLOR_ATTACHMENT8_NV     = 0x8CE8,
		GL_COLOR_ATTACHMENT9_NV     = 0x8CE9,
		GL_COLOR_ATTACHMENT10_NV    = 0x8CEA,
		GL_COLOR_ATTACHMENT11_NV    = 0x8CEB,
		GL_COLOR_ATTACHMENT12_NV    = 0x8CEC,
		GL_COLOR_ATTACHMENT13_NV    = 0x8CED,
		GL_COLOR_ATTACHMENT14_NV    = 0x8CEE,
		GL_COLOR_ATTACHMENT15_NV    = 0x8CEF;

}