/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_BufferUtils */

#ifndef _Included_org_lwjgl_BufferUtils
#define _Included_org_lwjgl_BufferUtils
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_BufferUtils
 * Method:    zeroBuffer0
 * Signature: (Ljava/nio/Buffer;JJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_BufferUtils_zeroBuffer0
  (JNIEnv *, jclass, jobject, jlong, jlong);

/*
 * Class:     org_lwjgl_BufferUtils
 * Method:    getBufferAddress
 * Signature: (Ljava/nio/Buffer;)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_BufferUtils_getBufferAddress
  (JNIEnv *, jclass, jobject);

#ifdef __cplusplus
}
#endif
#endif
