/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_LinuxCanvasImplementation */

#ifndef _Included_org_lwjgl_opengl_LinuxCanvasImplementation
#define _Included_org_lwjgl_opengl_LinuxCanvasImplementation
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_LinuxCanvasImplementation
 * Method:    nFindVisualIDFromFormat
 * Signature: (JILorg/lwjgl/opengl/PixelFormat;)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxCanvasImplementation_nFindVisualIDFromFormat
  (JNIEnv *, jclass, jlong, jint, jobject);

#ifdef __cplusplus
}
#endif
#endif
