/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_LinuxSysImplementation */

#ifndef _Included_org_lwjgl_LinuxSysImplementation
#define _Included_org_lwjgl_LinuxSysImplementation
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_LinuxSysImplementation_JNI_VERSION
#define org_lwjgl_LinuxSysImplementation_JNI_VERSION 19L
#ifdef __cplusplus
}
#endif
#endif
