/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_LinuxPbufferPeerInfo */

#ifndef _Included_org_lwjgl_opengl_LinuxPbufferPeerInfo
#define _Included_org_lwjgl_opengl_LinuxPbufferPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_LinuxPbufferPeerInfo
 * Method:    nInitHandle
 * Signature: (JILjava/nio/ByteBuffer;IILorg/lwjgl/opengl/PixelFormat;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxPbufferPeerInfo_nInitHandle
  (JNIEnv *, jclass, jlong, jint, jobject, jint, jint, jobject);

/*
 * Class:     org_lwjgl_opengl_LinuxPbufferPeerInfo
 * Method:    nDestroy
 * Signature: (Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxPbufferPeerInfo_nDestroy
  (JNIEnv *, jclass, jobject);

#ifdef __cplusplus
}
#endif
#endif
