/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_MacOSXMouseEventQueue */

#ifndef _Included_org_lwjgl_opengl_MacOSXMouseEventQueue
#define _Included_org_lwjgl_opengl_MacOSXMouseEventQueue
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengl_MacOSXMouseEventQueue_QUEUE_SIZE
#define org_lwjgl_opengl_MacOSXMouseEventQueue_QUEUE_SIZE 200L
#undef org_lwjgl_opengl_MacOSXMouseEventQueue_WHEEL_SCALE
#define org_lwjgl_opengl_MacOSXMouseEventQueue_WHEEL_SCALE 120L
#undef org_lwjgl_opengl_MacOSXMouseEventQueue_NUM_BUTTONS
#define org_lwjgl_opengl_MacOSXMouseEventQueue_NUM_BUTTONS 3L
/*
 * Class:     org_lwjgl_opengl_MacOSXMouseEventQueue
 * Method:    getMouseDeltas
 * Signature: (Ljava/nio/IntBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXMouseEventQueue_getMouseDeltas
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_MacOSXMouseEventQueue
 * Method:    nWarpCursor
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXMouseEventQueue_nWarpCursor
  (JNIEnv *, jclass, jint, jint);

/*
 * Class:     org_lwjgl_opengl_MacOSXMouseEventQueue
 * Method:    nGrabMouse
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXMouseEventQueue_nGrabMouse
  (JNIEnv *, jclass, jboolean);

#ifdef __cplusplus
}
#endif
#endif
