/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_LinuxDisplayPeerInfo */

#ifndef _Included_org_lwjgl_opengl_LinuxDisplayPeerInfo
#define _Included_org_lwjgl_opengl_LinuxDisplayPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_LinuxDisplayPeerInfo
 * Method:    initDefaultPeerInfo
 * Signature: (JILjava/nio/ByteBuffer;Lorg/lwjgl/opengl/PixelFormat;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplayPeerInfo_initDefaultPeerInfo
  (JNIEnv *, jclass, jlong, jint, jobject, jobject);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplayPeerInfo
 * Method:    initDrawable
 * Signature: (JLjava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplayPeerInfo_initDrawable
  (JNIEnv *, jclass, jlong, jobject);

#ifdef __cplusplus
}
#endif
#endif
