/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_WindowsPbufferPeerInfo */

#ifndef _Included_org_lwjgl_opengl_WindowsPbufferPeerInfo
#define _Included_org_lwjgl_opengl_WindowsPbufferPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_WindowsPbufferPeerInfo
 * Method:    nCreate
 * Signature: (Ljava/nio/ByteBuffer;IILorg/lwjgl/opengl/PixelFormat;Ljava/nio/IntBuffer;Ljava/nio/IntBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsPbufferPeerInfo_nCreate
  (JNIEnv *, jclass, jobject, jint, jint, jobject, jobject, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsPbufferPeerInfo
 * Method:    nIsBufferLost
 * Signature: (Ljava/nio/ByteBuffer;)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_WindowsPbufferPeerInfo_nIsBufferLost
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsPbufferPeerInfo
 * Method:    nSetPbufferAttrib
 * Signature: (Ljava/nio/ByteBuffer;II)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsPbufferPeerInfo_nSetPbufferAttrib
  (JNIEnv *, jclass, jobject, jint, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsPbufferPeerInfo
 * Method:    nBindTexImageToPbuffer
 * Signature: (Ljava/nio/ByteBuffer;I)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsPbufferPeerInfo_nBindTexImageToPbuffer
  (JNIEnv *, jclass, jobject, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsPbufferPeerInfo
 * Method:    nReleaseTexImageFromPbuffer
 * Signature: (Ljava/nio/ByteBuffer;I)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsPbufferPeerInfo_nReleaseTexImageFromPbuffer
  (JNIEnv *, jclass, jobject, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsPbufferPeerInfo
 * Method:    nDestroy
 * Signature: (Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsPbufferPeerInfo_nDestroy
  (JNIEnv *, jclass, jobject);

#ifdef __cplusplus
}
#endif
#endif
