/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_input_Keyboard_KeyEvent */

#ifndef _Included_org_lwjgl_input_Keyboard_KeyEvent
#define _Included_org_lwjgl_input_Keyboard_KeyEvent
#ifdef __cplusplus
extern "C" {
#endif
#ifdef __cplusplus
}
#endif
#endif
