/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengles_EGLKHRFenceSync */

#ifndef _Included_org_lwjgl_opengles_EGLKHRFenceSync
#define _Included_org_lwjgl_opengles_EGLKHRFenceSync
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_FENCE_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_FENCE_KHR 12537L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_TYPE_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_TYPE_KHR 12535L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_STATUS_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_STATUS_KHR 12529L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_CONDITION_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_CONDITION_KHR 12536L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_SIGNALED_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_SIGNALED_KHR 12530L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_UNSIGNALED_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_UNSIGNALED_KHR 12531L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_PRIOR_COMMANDS_COMPLETE_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_PRIOR_COMMANDS_COMPLETE_KHR 12528L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_FLUSH_COMMANDS_BIT_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_SYNC_FLUSH_COMMANDS_BIT_KHR 1L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_FOREVER_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_FOREVER_KHR -1i64
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_TIMEOUT_EXPIRED_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_TIMEOUT_EXPIRED_KHR 12533L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_CONDITION_SATISFIED_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_CONDITION_SATISFIED_KHR 12534L
#undef org_lwjgl_opengles_EGLKHRFenceSync_EGL_NO_SYNC_KHR
#define org_lwjgl_opengles_EGLKHRFenceSync_EGL_NO_SYNC_KHR 0i64
/*
 * Class:     org_lwjgl_opengles_EGLKHRFenceSync
 * Method:    initNativeStubs
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengles_EGLKHRFenceSync_initNativeStubs
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGLKHRFenceSync
 * Method:    neglCreateSyncKHR
 * Signature: (JIJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGLKHRFenceSync_neglCreateSyncKHR
  (JNIEnv *, jclass, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGLKHRFenceSync
 * Method:    neglDestroySyncKHR
 * Signature: (JJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGLKHRFenceSync_neglDestroySyncKHR
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGLKHRFenceSync
 * Method:    neglClientWaitSyncKHR
 * Signature: (JJIJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengles_EGLKHRFenceSync_neglClientWaitSyncKHR
  (JNIEnv *, jclass, jlong, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGLKHRFenceSync
 * Method:    neglGetSyncAttribKHR
 * Signature: (JJIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGLKHRFenceSync_neglGetSyncAttribKHR
  (JNIEnv *, jclass, jlong, jlong, jint, jlong);

#ifdef __cplusplus
}
#endif
#endif
