/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengles_EGLNVSync */

#ifndef _Included_org_lwjgl_opengles_EGLNVSync
#define _Included_org_lwjgl_opengles_EGLNVSync
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengles_EGLNVSync_EGL_SYNC_PRIOR_COMMANDS_COMPLETE_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_SYNC_PRIOR_COMMANDS_COMPLETE_NV 12518L
#undef org_lwjgl_opengles_EGLNVSync_EGL_SYNC_STATUS_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_SYNC_STATUS_NV 12519L
#undef org_lwjgl_opengles_EGLNVSync_EGL_SIGNALED_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_SIGNALED_NV 12520L
#undef org_lwjgl_opengles_EGLNVSync_EGL_UNSIGNALED_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_UNSIGNALED_NV 12521L
#undef org_lwjgl_opengles_EGLNVSync_EGL_SYNC_FLUSH_COMMANDS_BIT_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_SYNC_FLUSH_COMMANDS_BIT_NV 1L
#undef org_lwjgl_opengles_EGLNVSync_EGL_FOREVER_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_FOREVER_NV -1i64
#undef org_lwjgl_opengles_EGLNVSync_EGL_ALREADY_SIGNALED_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_ALREADY_SIGNALED_NV 12522L
#undef org_lwjgl_opengles_EGLNVSync_EGL_TIMEOUT_EXPIRED_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_TIMEOUT_EXPIRED_NV 12523L
#undef org_lwjgl_opengles_EGLNVSync_EGL_CONDITION_SATISFIED_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_CONDITION_SATISFIED_NV 12524L
#undef org_lwjgl_opengles_EGLNVSync_EGL_SYNC_TYPE_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_SYNC_TYPE_NV 12525L
#undef org_lwjgl_opengles_EGLNVSync_EGL_SYNC_CONDITION_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_SYNC_CONDITION_NV 12526L
#undef org_lwjgl_opengles_EGLNVSync_EGL_SYNC_FENCE_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_SYNC_FENCE_NV 12527L
#undef org_lwjgl_opengles_EGLNVSync_EGL_NO_SYNC_NV
#define org_lwjgl_opengles_EGLNVSync_EGL_NO_SYNC_NV 0i64
/*
 * Class:     org_lwjgl_opengles_EGLNVSync
 * Method:    initNativeStubs
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengles_EGLNVSync_initNativeStubs
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGLNVSync
 * Method:    neglCreateFenceSyncNV
 * Signature: (JIJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGLNVSync_neglCreateFenceSyncNV
  (JNIEnv *, jclass, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGLNVSync
 * Method:    neglDestroySyncNV
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGLNVSync_neglDestroySyncNV
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengles_EGLNVSync
 * Method:    neglFenceNV
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGLNVSync_neglFenceNV
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengles_EGLNVSync
 * Method:    neglClientWaitSyncNV
 * Signature: (JIJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengles_EGLNVSync_neglClientWaitSyncNV
  (JNIEnv *, jclass, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGLNVSync
 * Method:    neglSignalSyncNV
 * Signature: (JI)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGLNVSync_neglSignalSyncNV
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengles_EGLNVSync
 * Method:    neglGetSyncAttribNV
 * Signature: (JIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGLNVSync_neglGetSyncAttribNV
  (JNIEnv *, jclass, jlong, jint, jlong);

#ifdef __cplusplus
}
#endif
#endif
