/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_WindowsAWTGLCanvasPeerInfo */

#ifndef _Included_org_lwjgl_opengl_WindowsAWTGLCanvasPeerInfo
#define _Included_org_lwjgl_opengl_WindowsAWTGLCanvasPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_WindowsAWTGLCanvasPeerInfo
 * Method:    nInitHandle
 * Signature: (Ljava/nio/ByteBuffer;Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsAWTGLCanvasPeerInfo_nInitHandle
  (JNIEnv *, jclass, jobject, jobject);

#ifdef __cplusplus
}
#endif
#endif
