/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_openal_AL */

#ifndef _Included_org_lwjgl_openal_AL
#define _Included_org_lwjgl_openal_AL
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_openal_AL
 * Method:    nCreate
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_openal_AL_nCreate
  (JNIEnv *, jclass, jstring);

/*
 * Class:     org_lwjgl_openal_AL
 * Method:    nCreateDefault
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_openal_AL_nCreateDefault
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_openal_AL
 * Method:    nDestroy
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_openal_AL_nDestroy
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_openal_AL
 * Method:    resetNativeStubs
 * Signature: (Ljava/lang/Class;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_openal_AL_resetNativeStubs
  (JNIEnv *, jclass, jclass);

#ifdef __cplusplus
}
#endif
#endif
