/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_input_Cursor */

#ifndef _Included_org_lwjgl_input_Cursor
#define _Included_org_lwjgl_input_Cursor
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_input_Cursor_CURSOR_ONE_BIT_TRANSPARENCY
#define org_lwjgl_input_Cursor_CURSOR_ONE_BIT_TRANSPARENCY 1L
#undef org_lwjgl_input_Cursor_CURSOR_8_BIT_ALPHA
#define org_lwjgl_input_Cursor_CURSOR_8_BIT_ALPHA 2L
#undef org_lwjgl_input_Cursor_CURSOR_ANIMATION
#define org_lwjgl_input_Cursor_CURSOR_ANIMATION 4L
#ifdef __cplusplus
}
#endif
#endif
