/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_CallbackUtil */

#ifndef _Included_org_lwjgl_opengl_CallbackUtil
#define _Included_org_lwjgl_opengl_CallbackUtil
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_CallbackUtil
 * Method:    ncreateGlobalRef
 * Signature: (Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_CallbackUtil_ncreateGlobalRef
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_CallbackUtil
 * Method:    deleteGlobalRef
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_CallbackUtil_deleteGlobalRef
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_CallbackUtil
 * Method:    getDebugOutputCallbackARB
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_CallbackUtil_getDebugOutputCallbackARB
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_CallbackUtil
 * Method:    getDebugOutputCallbackAMD
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_CallbackUtil_getDebugOutputCallbackAMD
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_CallbackUtil
 * Method:    getDebugCallbackKHR
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_CallbackUtil_getDebugCallbackKHR
  (JNIEnv *, jclass);

#ifdef __cplusplus
}
#endif
#endif
