/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_NVVideoCaptureUtil */

#ifndef _Included_org_lwjgl_opengl_NVVideoCaptureUtil
#define _Included_org_lwjgl_opengl_NVVideoCaptureUtil
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_NVVideoCaptureUtil
 * Method:    nglBindVideoCaptureDeviceNV
 * Signature: (Ljava/nio/ByteBuffer;IJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_NVVideoCaptureUtil_nglBindVideoCaptureDeviceNV
  (JNIEnv *, jclass, jobject, jint, jlong);

/*
 * Class:     org_lwjgl_opengl_NVVideoCaptureUtil
 * Method:    nglEnumerateVideoCaptureDevicesNV
 * Signature: (Ljava/nio/ByteBuffer;Ljava/nio/LongBuffer;I)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_NVVideoCaptureUtil_nglEnumerateVideoCaptureDevicesNV
  (JNIEnv *, jclass, jobject, jobject, jint);

/*
 * Class:     org_lwjgl_opengl_NVVideoCaptureUtil
 * Method:    nglLockVideoCaptureDeviceNV
 * Signature: (Ljava/nio/ByteBuffer;J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_NVVideoCaptureUtil_nglLockVideoCaptureDeviceNV
  (JNIEnv *, jclass, jobject, jlong);

/*
 * Class:     org_lwjgl_opengl_NVVideoCaptureUtil
 * Method:    nglQueryVideoCaptureDeviceNV
 * Signature: (Ljava/nio/ByteBuffer;JILjava/nio/IntBuffer;I)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_NVVideoCaptureUtil_nglQueryVideoCaptureDeviceNV
  (JNIEnv *, jclass, jobject, jlong, jint, jobject, jint);

/*
 * Class:     org_lwjgl_opengl_NVVideoCaptureUtil
 * Method:    nglReleaseVideoCaptureDeviceNV
 * Signature: (Ljava/nio/ByteBuffer;J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_NVVideoCaptureUtil_nglReleaseVideoCaptureDeviceNV
  (JNIEnv *, jclass, jobject, jlong);

#ifdef __cplusplus
}
#endif
#endif
