/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_LinuxAWTGLCanvasPeerInfo */

#ifndef _Included_org_lwjgl_opengl_LinuxAWTGLCanvasPeerInfo
#define _Included_org_lwjgl_opengl_LinuxAWTGLCanvasPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_LinuxAWTGLCanvasPeerInfo
 * Method:    getScreenFromSurfaceInfo
 * Signature: (Ljava/nio/ByteBuffer;)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxAWTGLCanvasPeerInfo_getScreenFromSurfaceInfo
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_LinuxAWTGLCanvasPeerInfo
 * Method:    nInitHandle
 * Signature: (ILjava/nio/ByteBuffer;Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxAWTGLCanvasPeerInfo_nInitHandle
  (JNIEnv *, jclass, jint, jobject, jobject);

#ifdef __cplusplus
}
#endif
#endif
