/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_MacOSXPeerInfo */

#ifndef _Included_org_lwjgl_opengl_MacOSXPeerInfo
#define _Included_org_lwjgl_opengl_MacOSXPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_MacOSXPeerInfo
 * Method:    createHandle
 * Signature: ()Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_MacOSXPeerInfo_createHandle
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_MacOSXPeerInfo
 * Method:    nChoosePixelFormat
 * Signature: (Ljava/nio/ByteBuffer;Lorg/lwjgl/opengl/PixelFormat;ZZZZZ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXPeerInfo_nChoosePixelFormat
  (JNIEnv *, jclass, jobject, jobject, jboolean, jboolean, jboolean, jboolean, jboolean);

/*
 * Class:     org_lwjgl_opengl_MacOSXPeerInfo
 * Method:    nDestroy
 * Signature: (Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXPeerInfo_nDestroy
  (JNIEnv *, jclass, jobject);

#ifdef __cplusplus
}
#endif
#endif
