/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_DefaultSysImplementation */

#ifndef _Included_org_lwjgl_DefaultSysImplementation
#define _Included_org_lwjgl_DefaultSysImplementation
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_DefaultSysImplementation
 * Method:    getJNIVersion
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_DefaultSysImplementation_getJNIVersion
  (JNIEnv *, jobject);

/*
 * Class:     org_lwjgl_DefaultSysImplementation
 * Method:    getPointerSize
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_DefaultSysImplementation_getPointerSize
  (JNIEnv *, jobject);

/*
 * Class:     org_lwjgl_DefaultSysImplementation
 * Method:    setDebug
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_DefaultSysImplementation_setDebug
  (JNIEnv *, jobject, jboolean);

#ifdef __cplusplus
}
#endif
#endif
