/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_WindowsDisplayPeerInfo */

#ifndef _Included_org_lwjgl_opengl_WindowsDisplayPeerInfo
#define _Included_org_lwjgl_opengl_WindowsDisplayPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_WindowsDisplayPeerInfo
 * Method:    nInitDC
 * Signature: (Ljava/nio/ByteBuffer;JJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplayPeerInfo_nInitDC
  (JNIEnv *, jclass, jobject, jlong, jlong);

#ifdef __cplusplus
}
#endif
#endif
