/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_input_Keyboard */

#ifndef _Included_org_lwjgl_input_Keyboard
#define _Included_org_lwjgl_input_Keyboard
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_input_Keyboard_EVENT_SIZE
#define org_lwjgl_input_Keyboard_EVENT_SIZE 18L
#undef org_lwjgl_input_Keyboard_CHAR_NONE
#define org_lwjgl_input_Keyboard_CHAR_NONE 0L
#undef org_lwjgl_input_Keyboard_KEY_NONE
#define org_lwjgl_input_Keyboard_KEY_NONE 0L
#undef org_lwjgl_input_Keyboard_KEY_ESCAPE
#define org_lwjgl_input_Keyboard_KEY_ESCAPE 1L
#undef org_lwjgl_input_Keyboard_KEY_1
#define org_lwjgl_input_Keyboard_KEY_1 2L
#undef org_lwjgl_input_Keyboard_KEY_2
#define org_lwjgl_input_Keyboard_KEY_2 3L
#undef org_lwjgl_input_Keyboard_KEY_3
#define org_lwjgl_input_Keyboard_KEY_3 4L
#undef org_lwjgl_input_Keyboard_KEY_4
#define org_lwjgl_input_Keyboard_KEY_4 5L
#undef org_lwjgl_input_Keyboard_KEY_5
#define org_lwjgl_input_Keyboard_KEY_5 6L
#undef org_lwjgl_input_Keyboard_KEY_6
#define org_lwjgl_input_Keyboard_KEY_6 7L
#undef org_lwjgl_input_Keyboard_KEY_7
#define org_lwjgl_input_Keyboard_KEY_7 8L
#undef org_lwjgl_input_Keyboard_KEY_8
#define org_lwjgl_input_Keyboard_KEY_8 9L
#undef org_lwjgl_input_Keyboard_KEY_9
#define org_lwjgl_input_Keyboard_KEY_9 10L
#undef org_lwjgl_input_Keyboard_KEY_0
#define org_lwjgl_input_Keyboard_KEY_0 11L
#undef org_lwjgl_input_Keyboard_KEY_MINUS
#define org_lwjgl_input_Keyboard_KEY_MINUS 12L
#undef org_lwjgl_input_Keyboard_KEY_EQUALS
#define org_lwjgl_input_Keyboard_KEY_EQUALS 13L
#undef org_lwjgl_input_Keyboard_KEY_BACK
#define org_lwjgl_input_Keyboard_KEY_BACK 14L
#undef org_lwjgl_input_Keyboard_KEY_TAB
#define org_lwjgl_input_Keyboard_KEY_TAB 15L
#undef org_lwjgl_input_Keyboard_KEY_Q
#define org_lwjgl_input_Keyboard_KEY_Q 16L
#undef org_lwjgl_input_Keyboard_KEY_W
#define org_lwjgl_input_Keyboard_KEY_W 17L
#undef org_lwjgl_input_Keyboard_KEY_E
#define org_lwjgl_input_Keyboard_KEY_E 18L
#undef org_lwjgl_input_Keyboard_KEY_R
#define org_lwjgl_input_Keyboard_KEY_R 19L
#undef org_lwjgl_input_Keyboard_KEY_T
#define org_lwjgl_input_Keyboard_KEY_T 20L
#undef org_lwjgl_input_Keyboard_KEY_Y
#define org_lwjgl_input_Keyboard_KEY_Y 21L
#undef org_lwjgl_input_Keyboard_KEY_U
#define org_lwjgl_input_Keyboard_KEY_U 22L
#undef org_lwjgl_input_Keyboard_KEY_I
#define org_lwjgl_input_Keyboard_KEY_I 23L
#undef org_lwjgl_input_Keyboard_KEY_O
#define org_lwjgl_input_Keyboard_KEY_O 24L
#undef org_lwjgl_input_Keyboard_KEY_P
#define org_lwjgl_input_Keyboard_KEY_P 25L
#undef org_lwjgl_input_Keyboard_KEY_LBRACKET
#define org_lwjgl_input_Keyboard_KEY_LBRACKET 26L
#undef org_lwjgl_input_Keyboard_KEY_RBRACKET
#define org_lwjgl_input_Keyboard_KEY_RBRACKET 27L
#undef org_lwjgl_input_Keyboard_KEY_RETURN
#define org_lwjgl_input_Keyboard_KEY_RETURN 28L
#undef org_lwjgl_input_Keyboard_KEY_LCONTROL
#define org_lwjgl_input_Keyboard_KEY_LCONTROL 29L
#undef org_lwjgl_input_Keyboard_KEY_A
#define org_lwjgl_input_Keyboard_KEY_A 30L
#undef org_lwjgl_input_Keyboard_KEY_S
#define org_lwjgl_input_Keyboard_KEY_S 31L
#undef org_lwjgl_input_Keyboard_KEY_D
#define org_lwjgl_input_Keyboard_KEY_D 32L
#undef org_lwjgl_input_Keyboard_KEY_F
#define org_lwjgl_input_Keyboard_KEY_F 33L
#undef org_lwjgl_input_Keyboard_KEY_G
#define org_lwjgl_input_Keyboard_KEY_G 34L
#undef org_lwjgl_input_Keyboard_KEY_H
#define org_lwjgl_input_Keyboard_KEY_H 35L
#undef org_lwjgl_input_Keyboard_KEY_J
#define org_lwjgl_input_Keyboard_KEY_J 36L
#undef org_lwjgl_input_Keyboard_KEY_K
#define org_lwjgl_input_Keyboard_KEY_K 37L
#undef org_lwjgl_input_Keyboard_KEY_L
#define org_lwjgl_input_Keyboard_KEY_L 38L
#undef org_lwjgl_input_Keyboard_KEY_SEMICOLON
#define org_lwjgl_input_Keyboard_KEY_SEMICOLON 39L
#undef org_lwjgl_input_Keyboard_KEY_APOSTROPHE
#define org_lwjgl_input_Keyboard_KEY_APOSTROPHE 40L
#undef org_lwjgl_input_Keyboard_KEY_GRAVE
#define org_lwjgl_input_Keyboard_KEY_GRAVE 41L
#undef org_lwjgl_input_Keyboard_KEY_LSHIFT
#define org_lwjgl_input_Keyboard_KEY_LSHIFT 42L
#undef org_lwjgl_input_Keyboard_KEY_BACKSLASH
#define org_lwjgl_input_Keyboard_KEY_BACKSLASH 43L
#undef org_lwjgl_input_Keyboard_KEY_Z
#define org_lwjgl_input_Keyboard_KEY_Z 44L
#undef org_lwjgl_input_Keyboard_KEY_X
#define org_lwjgl_input_Keyboard_KEY_X 45L
#undef org_lwjgl_input_Keyboard_KEY_C
#define org_lwjgl_input_Keyboard_KEY_C 46L
#undef org_lwjgl_input_Keyboard_KEY_V
#define org_lwjgl_input_Keyboard_KEY_V 47L
#undef org_lwjgl_input_Keyboard_KEY_B
#define org_lwjgl_input_Keyboard_KEY_B 48L
#undef org_lwjgl_input_Keyboard_KEY_N
#define org_lwjgl_input_Keyboard_KEY_N 49L
#undef org_lwjgl_input_Keyboard_KEY_M
#define org_lwjgl_input_Keyboard_KEY_M 50L
#undef org_lwjgl_input_Keyboard_KEY_COMMA
#define org_lwjgl_input_Keyboard_KEY_COMMA 51L
#undef org_lwjgl_input_Keyboard_KEY_PERIOD
#define org_lwjgl_input_Keyboard_KEY_PERIOD 52L
#undef org_lwjgl_input_Keyboard_KEY_SLASH
#define org_lwjgl_input_Keyboard_KEY_SLASH 53L
#undef org_lwjgl_input_Keyboard_KEY_RSHIFT
#define org_lwjgl_input_Keyboard_KEY_RSHIFT 54L
#undef org_lwjgl_input_Keyboard_KEY_MULTIPLY
#define org_lwjgl_input_Keyboard_KEY_MULTIPLY 55L
#undef org_lwjgl_input_Keyboard_KEY_LMENU
#define org_lwjgl_input_Keyboard_KEY_LMENU 56L
#undef org_lwjgl_input_Keyboard_KEY_SPACE
#define org_lwjgl_input_Keyboard_KEY_SPACE 57L
#undef org_lwjgl_input_Keyboard_KEY_CAPITAL
#define org_lwjgl_input_Keyboard_KEY_CAPITAL 58L
#undef org_lwjgl_input_Keyboard_KEY_F1
#define org_lwjgl_input_Keyboard_KEY_F1 59L
#undef org_lwjgl_input_Keyboard_KEY_F2
#define org_lwjgl_input_Keyboard_KEY_F2 60L
#undef org_lwjgl_input_Keyboard_KEY_F3
#define org_lwjgl_input_Keyboard_KEY_F3 61L
#undef org_lwjgl_input_Keyboard_KEY_F4
#define org_lwjgl_input_Keyboard_KEY_F4 62L
#undef org_lwjgl_input_Keyboard_KEY_F5
#define org_lwjgl_input_Keyboard_KEY_F5 63L
#undef org_lwjgl_input_Keyboard_KEY_F6
#define org_lwjgl_input_Keyboard_KEY_F6 64L
#undef org_lwjgl_input_Keyboard_KEY_F7
#define org_lwjgl_input_Keyboard_KEY_F7 65L
#undef org_lwjgl_input_Keyboard_KEY_F8
#define org_lwjgl_input_Keyboard_KEY_F8 66L
#undef org_lwjgl_input_Keyboard_KEY_F9
#define org_lwjgl_input_Keyboard_KEY_F9 67L
#undef org_lwjgl_input_Keyboard_KEY_F10
#define org_lwjgl_input_Keyboard_KEY_F10 68L
#undef org_lwjgl_input_Keyboard_KEY_NUMLOCK
#define org_lwjgl_input_Keyboard_KEY_NUMLOCK 69L
#undef org_lwjgl_input_Keyboard_KEY_SCROLL
#define org_lwjgl_input_Keyboard_KEY_SCROLL 70L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD7
#define org_lwjgl_input_Keyboard_KEY_NUMPAD7 71L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD8
#define org_lwjgl_input_Keyboard_KEY_NUMPAD8 72L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD9
#define org_lwjgl_input_Keyboard_KEY_NUMPAD9 73L
#undef org_lwjgl_input_Keyboard_KEY_SUBTRACT
#define org_lwjgl_input_Keyboard_KEY_SUBTRACT 74L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD4
#define org_lwjgl_input_Keyboard_KEY_NUMPAD4 75L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD5
#define org_lwjgl_input_Keyboard_KEY_NUMPAD5 76L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD6
#define org_lwjgl_input_Keyboard_KEY_NUMPAD6 77L
#undef org_lwjgl_input_Keyboard_KEY_ADD
#define org_lwjgl_input_Keyboard_KEY_ADD 78L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD1
#define org_lwjgl_input_Keyboard_KEY_NUMPAD1 79L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD2
#define org_lwjgl_input_Keyboard_KEY_NUMPAD2 80L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD3
#define org_lwjgl_input_Keyboard_KEY_NUMPAD3 81L
#undef org_lwjgl_input_Keyboard_KEY_NUMPAD0
#define org_lwjgl_input_Keyboard_KEY_NUMPAD0 82L
#undef org_lwjgl_input_Keyboard_KEY_DECIMAL
#define org_lwjgl_input_Keyboard_KEY_DECIMAL 83L
#undef org_lwjgl_input_Keyboard_KEY_F11
#define org_lwjgl_input_Keyboard_KEY_F11 87L
#undef org_lwjgl_input_Keyboard_KEY_F12
#define org_lwjgl_input_Keyboard_KEY_F12 88L
#undef org_lwjgl_input_Keyboard_KEY_F13
#define org_lwjgl_input_Keyboard_KEY_F13 100L
#undef org_lwjgl_input_Keyboard_KEY_F14
#define org_lwjgl_input_Keyboard_KEY_F14 101L
#undef org_lwjgl_input_Keyboard_KEY_F15
#define org_lwjgl_input_Keyboard_KEY_F15 102L
#undef org_lwjgl_input_Keyboard_KEY_F16
#define org_lwjgl_input_Keyboard_KEY_F16 103L
#undef org_lwjgl_input_Keyboard_KEY_F17
#define org_lwjgl_input_Keyboard_KEY_F17 104L
#undef org_lwjgl_input_Keyboard_KEY_F18
#define org_lwjgl_input_Keyboard_KEY_F18 105L
#undef org_lwjgl_input_Keyboard_KEY_KANA
#define org_lwjgl_input_Keyboard_KEY_KANA 112L
#undef org_lwjgl_input_Keyboard_KEY_F19
#define org_lwjgl_input_Keyboard_KEY_F19 113L
#undef org_lwjgl_input_Keyboard_KEY_CONVERT
#define org_lwjgl_input_Keyboard_KEY_CONVERT 121L
#undef org_lwjgl_input_Keyboard_KEY_NOCONVERT
#define org_lwjgl_input_Keyboard_KEY_NOCONVERT 123L
#undef org_lwjgl_input_Keyboard_KEY_YEN
#define org_lwjgl_input_Keyboard_KEY_YEN 125L
#undef org_lwjgl_input_Keyboard_KEY_NUMPADEQUALS
#define org_lwjgl_input_Keyboard_KEY_NUMPADEQUALS 141L
#undef org_lwjgl_input_Keyboard_KEY_CIRCUMFLEX
#define org_lwjgl_input_Keyboard_KEY_CIRCUMFLEX 144L
#undef org_lwjgl_input_Keyboard_KEY_AT
#define org_lwjgl_input_Keyboard_KEY_AT 145L
#undef org_lwjgl_input_Keyboard_KEY_COLON
#define org_lwjgl_input_Keyboard_KEY_COLON 146L
#undef org_lwjgl_input_Keyboard_KEY_UNDERLINE
#define org_lwjgl_input_Keyboard_KEY_UNDERLINE 147L
#undef org_lwjgl_input_Keyboard_KEY_KANJI
#define org_lwjgl_input_Keyboard_KEY_KANJI 148L
#undef org_lwjgl_input_Keyboard_KEY_STOP
#define org_lwjgl_input_Keyboard_KEY_STOP 149L
#undef org_lwjgl_input_Keyboard_KEY_AX
#define org_lwjgl_input_Keyboard_KEY_AX 150L
#undef org_lwjgl_input_Keyboard_KEY_UNLABELED
#define org_lwjgl_input_Keyboard_KEY_UNLABELED 151L
#undef org_lwjgl_input_Keyboard_KEY_NUMPADENTER
#define org_lwjgl_input_Keyboard_KEY_NUMPADENTER 156L
#undef org_lwjgl_input_Keyboard_KEY_RCONTROL
#define org_lwjgl_input_Keyboard_KEY_RCONTROL 157L
#undef org_lwjgl_input_Keyboard_KEY_SECTION
#define org_lwjgl_input_Keyboard_KEY_SECTION 167L
#undef org_lwjgl_input_Keyboard_KEY_NUMPADCOMMA
#define org_lwjgl_input_Keyboard_KEY_NUMPADCOMMA 179L
#undef org_lwjgl_input_Keyboard_KEY_DIVIDE
#define org_lwjgl_input_Keyboard_KEY_DIVIDE 181L
#undef org_lwjgl_input_Keyboard_KEY_SYSRQ
#define org_lwjgl_input_Keyboard_KEY_SYSRQ 183L
#undef org_lwjgl_input_Keyboard_KEY_RMENU
#define org_lwjgl_input_Keyboard_KEY_RMENU 184L
#undef org_lwjgl_input_Keyboard_KEY_FUNCTION
#define org_lwjgl_input_Keyboard_KEY_FUNCTION 196L
#undef org_lwjgl_input_Keyboard_KEY_PAUSE
#define org_lwjgl_input_Keyboard_KEY_PAUSE 197L
#undef org_lwjgl_input_Keyboard_KEY_HOME
#define org_lwjgl_input_Keyboard_KEY_HOME 199L
#undef org_lwjgl_input_Keyboard_KEY_UP
#define org_lwjgl_input_Keyboard_KEY_UP 200L
#undef org_lwjgl_input_Keyboard_KEY_PRIOR
#define org_lwjgl_input_Keyboard_KEY_PRIOR 201L
#undef org_lwjgl_input_Keyboard_KEY_LEFT
#define org_lwjgl_input_Keyboard_KEY_LEFT 203L
#undef org_lwjgl_input_Keyboard_KEY_RIGHT
#define org_lwjgl_input_Keyboard_KEY_RIGHT 205L
#undef org_lwjgl_input_Keyboard_KEY_END
#define org_lwjgl_input_Keyboard_KEY_END 207L
#undef org_lwjgl_input_Keyboard_KEY_DOWN
#define org_lwjgl_input_Keyboard_KEY_DOWN 208L
#undef org_lwjgl_input_Keyboard_KEY_NEXT
#define org_lwjgl_input_Keyboard_KEY_NEXT 209L
#undef org_lwjgl_input_Keyboard_KEY_INSERT
#define org_lwjgl_input_Keyboard_KEY_INSERT 210L
#undef org_lwjgl_input_Keyboard_KEY_DELETE
#define org_lwjgl_input_Keyboard_KEY_DELETE 211L
#undef org_lwjgl_input_Keyboard_KEY_CLEAR
#define org_lwjgl_input_Keyboard_KEY_CLEAR 218L
#undef org_lwjgl_input_Keyboard_KEY_LMETA
#define org_lwjgl_input_Keyboard_KEY_LMETA 219L
#undef org_lwjgl_input_Keyboard_KEY_LWIN
#define org_lwjgl_input_Keyboard_KEY_LWIN 219L
#undef org_lwjgl_input_Keyboard_KEY_RMETA
#define org_lwjgl_input_Keyboard_KEY_RMETA 220L
#undef org_lwjgl_input_Keyboard_KEY_RWIN
#define org_lwjgl_input_Keyboard_KEY_RWIN 220L
#undef org_lwjgl_input_Keyboard_KEY_APPS
#define org_lwjgl_input_Keyboard_KEY_APPS 221L
#undef org_lwjgl_input_Keyboard_KEY_POWER
#define org_lwjgl_input_Keyboard_KEY_POWER 222L
#undef org_lwjgl_input_Keyboard_KEY_SLEEP
#define org_lwjgl_input_Keyboard_KEY_SLEEP 223L
#undef org_lwjgl_input_Keyboard_KEYBOARD_SIZE
#define org_lwjgl_input_Keyboard_KEYBOARD_SIZE 256L
#undef org_lwjgl_input_Keyboard_BUFFER_SIZE
#define org_lwjgl_input_Keyboard_BUFFER_SIZE 50L
#ifdef __cplusplus
}
#endif
#endif
