/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengles_EGL */

#ifndef _Included_org_lwjgl_opengles_EGL
#define _Included_org_lwjgl_opengles_EGL
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengles_EGL_EGL_FALSE
#define org_lwjgl_opengles_EGL_EGL_FALSE 0L
#undef org_lwjgl_opengles_EGL_EGL_TRUE
#define org_lwjgl_opengles_EGL_EGL_TRUE 1L
#undef org_lwjgl_opengles_EGL_EGL_DEFAULT_DISPLAY
#define org_lwjgl_opengles_EGL_EGL_DEFAULT_DISPLAY 0L
#undef org_lwjgl_opengles_EGL_EGL_NO_CONTEXT
#define org_lwjgl_opengles_EGL_EGL_NO_CONTEXT 0L
#undef org_lwjgl_opengles_EGL_EGL_NO_DISPLAY
#define org_lwjgl_opengles_EGL_EGL_NO_DISPLAY 0L
#undef org_lwjgl_opengles_EGL_EGL_NO_SURFACE
#define org_lwjgl_opengles_EGL_EGL_NO_SURFACE 0L
#undef org_lwjgl_opengles_EGL_EGL_DONT_CARE
#define org_lwjgl_opengles_EGL_EGL_DONT_CARE -1L
#undef org_lwjgl_opengles_EGL_EGL_SUCCESS
#define org_lwjgl_opengles_EGL_EGL_SUCCESS 12288L
#undef org_lwjgl_opengles_EGL_EGL_NOT_INITIALIZED
#define org_lwjgl_opengles_EGL_EGL_NOT_INITIALIZED 12289L
#undef org_lwjgl_opengles_EGL_EGL_BAD_ACCESS
#define org_lwjgl_opengles_EGL_EGL_BAD_ACCESS 12290L
#undef org_lwjgl_opengles_EGL_EGL_BAD_ALLOC
#define org_lwjgl_opengles_EGL_EGL_BAD_ALLOC 12291L
#undef org_lwjgl_opengles_EGL_EGL_BAD_ATTRIBUTE
#define org_lwjgl_opengles_EGL_EGL_BAD_ATTRIBUTE 12292L
#undef org_lwjgl_opengles_EGL_EGL_BAD_CONFIG
#define org_lwjgl_opengles_EGL_EGL_BAD_CONFIG 12293L
#undef org_lwjgl_opengles_EGL_EGL_BAD_CONTEXT
#define org_lwjgl_opengles_EGL_EGL_BAD_CONTEXT 12294L
#undef org_lwjgl_opengles_EGL_EGL_BAD_CURRENT_SURFACE
#define org_lwjgl_opengles_EGL_EGL_BAD_CURRENT_SURFACE 12295L
#undef org_lwjgl_opengles_EGL_EGL_BAD_DISPLAY
#define org_lwjgl_opengles_EGL_EGL_BAD_DISPLAY 12296L
#undef org_lwjgl_opengles_EGL_EGL_BAD_MATCH
#define org_lwjgl_opengles_EGL_EGL_BAD_MATCH 12297L
#undef org_lwjgl_opengles_EGL_EGL_BAD_NATIVE_PIXMAP
#define org_lwjgl_opengles_EGL_EGL_BAD_NATIVE_PIXMAP 12298L
#undef org_lwjgl_opengles_EGL_EGL_BAD_NATIVE_WINDOW
#define org_lwjgl_opengles_EGL_EGL_BAD_NATIVE_WINDOW 12299L
#undef org_lwjgl_opengles_EGL_EGL_BAD_PARAMETER
#define org_lwjgl_opengles_EGL_EGL_BAD_PARAMETER 12300L
#undef org_lwjgl_opengles_EGL_EGL_BAD_SURFACE
#define org_lwjgl_opengles_EGL_EGL_BAD_SURFACE 12301L
#undef org_lwjgl_opengles_EGL_EGL_CONTEXT_LOST
#define org_lwjgl_opengles_EGL_EGL_CONTEXT_LOST 12302L
#undef org_lwjgl_opengles_EGL_EGL_BUFFER_SIZE
#define org_lwjgl_opengles_EGL_EGL_BUFFER_SIZE 12320L
#undef org_lwjgl_opengles_EGL_EGL_ALPHA_SIZE
#define org_lwjgl_opengles_EGL_EGL_ALPHA_SIZE 12321L
#undef org_lwjgl_opengles_EGL_EGL_BLUE_SIZE
#define org_lwjgl_opengles_EGL_EGL_BLUE_SIZE 12322L
#undef org_lwjgl_opengles_EGL_EGL_GREEN_SIZE
#define org_lwjgl_opengles_EGL_EGL_GREEN_SIZE 12323L
#undef org_lwjgl_opengles_EGL_EGL_RED_SIZE
#define org_lwjgl_opengles_EGL_EGL_RED_SIZE 12324L
#undef org_lwjgl_opengles_EGL_EGL_DEPTH_SIZE
#define org_lwjgl_opengles_EGL_EGL_DEPTH_SIZE 12325L
#undef org_lwjgl_opengles_EGL_EGL_STENCIL_SIZE
#define org_lwjgl_opengles_EGL_EGL_STENCIL_SIZE 12326L
#undef org_lwjgl_opengles_EGL_EGL_CONFIG_CAVEAT
#define org_lwjgl_opengles_EGL_EGL_CONFIG_CAVEAT 12327L
#undef org_lwjgl_opengles_EGL_EGL_CONFIG_ID
#define org_lwjgl_opengles_EGL_EGL_CONFIG_ID 12328L
#undef org_lwjgl_opengles_EGL_EGL_LEVEL
#define org_lwjgl_opengles_EGL_EGL_LEVEL 12329L
#undef org_lwjgl_opengles_EGL_EGL_MAX_PBUFFER_HEIGHT
#define org_lwjgl_opengles_EGL_EGL_MAX_PBUFFER_HEIGHT 12330L
#undef org_lwjgl_opengles_EGL_EGL_MAX_PBUFFER_PIXELS
#define org_lwjgl_opengles_EGL_EGL_MAX_PBUFFER_PIXELS 12331L
#undef org_lwjgl_opengles_EGL_EGL_MAX_PBUFFER_WIDTH
#define org_lwjgl_opengles_EGL_EGL_MAX_PBUFFER_WIDTH 12332L
#undef org_lwjgl_opengles_EGL_EGL_NATIVE_RENDERABLE
#define org_lwjgl_opengles_EGL_EGL_NATIVE_RENDERABLE 12333L
#undef org_lwjgl_opengles_EGL_EGL_NATIVE_VISUAL_ID
#define org_lwjgl_opengles_EGL_EGL_NATIVE_VISUAL_ID 12334L
#undef org_lwjgl_opengles_EGL_EGL_NATIVE_VISUAL_TYPE
#define org_lwjgl_opengles_EGL_EGL_NATIVE_VISUAL_TYPE 12335L
#undef org_lwjgl_opengles_EGL_EGL_SAMPLES
#define org_lwjgl_opengles_EGL_EGL_SAMPLES 12337L
#undef org_lwjgl_opengles_EGL_EGL_SAMPLE_BUFFERS
#define org_lwjgl_opengles_EGL_EGL_SAMPLE_BUFFERS 12338L
#undef org_lwjgl_opengles_EGL_EGL_SURFACE_TYPE
#define org_lwjgl_opengles_EGL_EGL_SURFACE_TYPE 12339L
#undef org_lwjgl_opengles_EGL_EGL_TRANSPARENT_TYPE
#define org_lwjgl_opengles_EGL_EGL_TRANSPARENT_TYPE 12340L
#undef org_lwjgl_opengles_EGL_EGL_TRANSPARENT_BLUE_VALUE
#define org_lwjgl_opengles_EGL_EGL_TRANSPARENT_BLUE_VALUE 12341L
#undef org_lwjgl_opengles_EGL_EGL_TRANSPARENT_GREEN_VALUE
#define org_lwjgl_opengles_EGL_EGL_TRANSPARENT_GREEN_VALUE 12342L
#undef org_lwjgl_opengles_EGL_EGL_TRANSPARENT_RED_VALUE
#define org_lwjgl_opengles_EGL_EGL_TRANSPARENT_RED_VALUE 12343L
#undef org_lwjgl_opengles_EGL_EGL_NONE
#define org_lwjgl_opengles_EGL_EGL_NONE 12344L
#undef org_lwjgl_opengles_EGL_EGL_BIND_TO_TEXTURE_RGB
#define org_lwjgl_opengles_EGL_EGL_BIND_TO_TEXTURE_RGB 12345L
#undef org_lwjgl_opengles_EGL_EGL_BIND_TO_TEXTURE_RGBA
#define org_lwjgl_opengles_EGL_EGL_BIND_TO_TEXTURE_RGBA 12346L
#undef org_lwjgl_opengles_EGL_EGL_MIN_SWAP_INTERVAL
#define org_lwjgl_opengles_EGL_EGL_MIN_SWAP_INTERVAL 12347L
#undef org_lwjgl_opengles_EGL_EGL_MAX_SWAP_INTERVAL
#define org_lwjgl_opengles_EGL_EGL_MAX_SWAP_INTERVAL 12348L
#undef org_lwjgl_opengles_EGL_EGL_LUMINANCE_SIZE
#define org_lwjgl_opengles_EGL_EGL_LUMINANCE_SIZE 12349L
#undef org_lwjgl_opengles_EGL_EGL_ALPHA_MASK_SIZE
#define org_lwjgl_opengles_EGL_EGL_ALPHA_MASK_SIZE 12350L
#undef org_lwjgl_opengles_EGL_EGL_COLOR_BUFFER_TYPE
#define org_lwjgl_opengles_EGL_EGL_COLOR_BUFFER_TYPE 12351L
#undef org_lwjgl_opengles_EGL_EGL_RENDERABLE_TYPE
#define org_lwjgl_opengles_EGL_EGL_RENDERABLE_TYPE 12352L
#undef org_lwjgl_opengles_EGL_EGL_MATCH_NATIVE_PIXMAP
#define org_lwjgl_opengles_EGL_EGL_MATCH_NATIVE_PIXMAP 12353L
#undef org_lwjgl_opengles_EGL_EGL_CONFORMANT
#define org_lwjgl_opengles_EGL_EGL_CONFORMANT 12354L
#undef org_lwjgl_opengles_EGL_EGL_SLOW_CONFIG
#define org_lwjgl_opengles_EGL_EGL_SLOW_CONFIG 12368L
#undef org_lwjgl_opengles_EGL_EGL_NON_CONFORMANT_CONFIG
#define org_lwjgl_opengles_EGL_EGL_NON_CONFORMANT_CONFIG 12369L
#undef org_lwjgl_opengles_EGL_EGL_TRANSPARENT_RGB
#define org_lwjgl_opengles_EGL_EGL_TRANSPARENT_RGB 12370L
#undef org_lwjgl_opengles_EGL_EGL_RGB_BUFFER
#define org_lwjgl_opengles_EGL_EGL_RGB_BUFFER 12430L
#undef org_lwjgl_opengles_EGL_EGL_LUMINANCE_BUFFER
#define org_lwjgl_opengles_EGL_EGL_LUMINANCE_BUFFER 12431L
#undef org_lwjgl_opengles_EGL_EGL_NO_TEXTURE
#define org_lwjgl_opengles_EGL_EGL_NO_TEXTURE 12380L
#undef org_lwjgl_opengles_EGL_EGL_TEXTURE_RGB
#define org_lwjgl_opengles_EGL_EGL_TEXTURE_RGB 12381L
#undef org_lwjgl_opengles_EGL_EGL_TEXTURE_RGBA
#define org_lwjgl_opengles_EGL_EGL_TEXTURE_RGBA 12382L
#undef org_lwjgl_opengles_EGL_EGL_TEXTURE_2D
#define org_lwjgl_opengles_EGL_EGL_TEXTURE_2D 12383L
#undef org_lwjgl_opengles_EGL_EGL_PBUFFER_BIT
#define org_lwjgl_opengles_EGL_EGL_PBUFFER_BIT 1L
#undef org_lwjgl_opengles_EGL_EGL_PIXMAP_BIT
#define org_lwjgl_opengles_EGL_EGL_PIXMAP_BIT 2L
#undef org_lwjgl_opengles_EGL_EGL_WINDOW_BIT
#define org_lwjgl_opengles_EGL_EGL_WINDOW_BIT 4L
#undef org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE_LINEAR_BIT
#define org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE_LINEAR_BIT 32L
#undef org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT_PRE_BIT
#define org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT_PRE_BIT 64L
#undef org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE_BOX_BIT
#define org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE_BOX_BIT 512L
#undef org_lwjgl_opengles_EGL_EGL_SWAP_BEHAVIOR_PRESERVED_BIT
#define org_lwjgl_opengles_EGL_EGL_SWAP_BEHAVIOR_PRESERVED_BIT 1024L
#undef org_lwjgl_opengles_EGL_EGL_OPENGL_ES_BIT
#define org_lwjgl_opengles_EGL_EGL_OPENGL_ES_BIT 1L
#undef org_lwjgl_opengles_EGL_EGL_OPENVG_BIT
#define org_lwjgl_opengles_EGL_EGL_OPENVG_BIT 2L
#undef org_lwjgl_opengles_EGL_EGL_OPENGL_ES2_BIT
#define org_lwjgl_opengles_EGL_EGL_OPENGL_ES2_BIT 4L
#undef org_lwjgl_opengles_EGL_EGL_OPENGL_BIT
#define org_lwjgl_opengles_EGL_EGL_OPENGL_BIT 8L
#undef org_lwjgl_opengles_EGL_EGL_VENDOR
#define org_lwjgl_opengles_EGL_EGL_VENDOR 12371L
#undef org_lwjgl_opengles_EGL_EGL_VERSION
#define org_lwjgl_opengles_EGL_EGL_VERSION 12372L
#undef org_lwjgl_opengles_EGL_EGL_EXTENSIONS
#define org_lwjgl_opengles_EGL_EGL_EXTENSIONS 12373L
#undef org_lwjgl_opengles_EGL_EGL_CLIENT_APIS
#define org_lwjgl_opengles_EGL_EGL_CLIENT_APIS 12429L
#undef org_lwjgl_opengles_EGL_EGL_HEIGHT
#define org_lwjgl_opengles_EGL_EGL_HEIGHT 12374L
#undef org_lwjgl_opengles_EGL_EGL_WIDTH
#define org_lwjgl_opengles_EGL_EGL_WIDTH 12375L
#undef org_lwjgl_opengles_EGL_EGL_LARGEST_PBUFFER
#define org_lwjgl_opengles_EGL_EGL_LARGEST_PBUFFER 12376L
#undef org_lwjgl_opengles_EGL_EGL_TEXTURE_FORMAT
#define org_lwjgl_opengles_EGL_EGL_TEXTURE_FORMAT 12416L
#undef org_lwjgl_opengles_EGL_EGL_TEXTURE_TARGET
#define org_lwjgl_opengles_EGL_EGL_TEXTURE_TARGET 12417L
#undef org_lwjgl_opengles_EGL_EGL_MIPMAP_TEXTURE
#define org_lwjgl_opengles_EGL_EGL_MIPMAP_TEXTURE 12418L
#undef org_lwjgl_opengles_EGL_EGL_MIPMAP_LEVEL
#define org_lwjgl_opengles_EGL_EGL_MIPMAP_LEVEL 12419L
#undef org_lwjgl_opengles_EGL_EGL_RENDER_BUFFER
#define org_lwjgl_opengles_EGL_EGL_RENDER_BUFFER 12422L
#undef org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE
#define org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE 12423L
#undef org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT
#define org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT 12424L
#undef org_lwjgl_opengles_EGL_EGL_HORIZONTAL_RESOLUTION
#define org_lwjgl_opengles_EGL_EGL_HORIZONTAL_RESOLUTION 12432L
#undef org_lwjgl_opengles_EGL_EGL_VERTICAL_RESOLUTION
#define org_lwjgl_opengles_EGL_EGL_VERTICAL_RESOLUTION 12433L
#undef org_lwjgl_opengles_EGL_EGL_PIXEL_ASPECT_RATIO
#define org_lwjgl_opengles_EGL_EGL_PIXEL_ASPECT_RATIO 12434L
#undef org_lwjgl_opengles_EGL_EGL_SWAP_BEHAVIOR
#define org_lwjgl_opengles_EGL_EGL_SWAP_BEHAVIOR 12435L
#undef org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE
#define org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE 12441L
#undef org_lwjgl_opengles_EGL_EGL_BACK_BUFFER
#define org_lwjgl_opengles_EGL_EGL_BACK_BUFFER 12420L
#undef org_lwjgl_opengles_EGL_EGL_SINGLE_BUFFER
#define org_lwjgl_opengles_EGL_EGL_SINGLE_BUFFER 12421L
#undef org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE_sRGB
#define org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE_sRGB 12425L
#undef org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE_LINEAR
#define org_lwjgl_opengles_EGL_EGL_VG_COLORSPACE_LINEAR 12426L
#undef org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT_NONPRE
#define org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT_NONPRE 12427L
#undef org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT_PRE
#define org_lwjgl_opengles_EGL_EGL_VG_ALPHA_FORMAT_PRE 12428L
#undef org_lwjgl_opengles_EGL_EGL_DISPLAY_SCALING
#define org_lwjgl_opengles_EGL_EGL_DISPLAY_SCALING 10000L
#undef org_lwjgl_opengles_EGL_EGL_UNKNOWN
#define org_lwjgl_opengles_EGL_EGL_UNKNOWN -1L
#undef org_lwjgl_opengles_EGL_EGL_BUFFER_PRESERVED
#define org_lwjgl_opengles_EGL_EGL_BUFFER_PRESERVED 12436L
#undef org_lwjgl_opengles_EGL_EGL_BUFFER_DESTROYED
#define org_lwjgl_opengles_EGL_EGL_BUFFER_DESTROYED 12437L
#undef org_lwjgl_opengles_EGL_EGL_OPENVG_IMAGE
#define org_lwjgl_opengles_EGL_EGL_OPENVG_IMAGE 12438L
#undef org_lwjgl_opengles_EGL_EGL_CONTEXT_CLIENT_TYPE
#define org_lwjgl_opengles_EGL_EGL_CONTEXT_CLIENT_TYPE 12439L
#undef org_lwjgl_opengles_EGL_EGL_CONTEXT_CLIENT_VERSION
#define org_lwjgl_opengles_EGL_EGL_CONTEXT_CLIENT_VERSION 12440L
#undef org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE_DEFAULT
#define org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE_DEFAULT 12442L
#undef org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE_BOX
#define org_lwjgl_opengles_EGL_EGL_MULTISAMPLE_RESOLVE_BOX 12443L
#undef org_lwjgl_opengles_EGL_EGL_OPENGL_ES_API
#define org_lwjgl_opengles_EGL_EGL_OPENGL_ES_API 12448L
#undef org_lwjgl_opengles_EGL_EGL_OPENVG_API
#define org_lwjgl_opengles_EGL_EGL_OPENVG_API 12449L
#undef org_lwjgl_opengles_EGL_EGL_OPENGL_API
#define org_lwjgl_opengles_EGL_EGL_OPENGL_API 12450L
#undef org_lwjgl_opengles_EGL_EGL_DRAW
#define org_lwjgl_opengles_EGL_EGL_DRAW 12377L
#undef org_lwjgl_opengles_EGL_EGL_READ
#define org_lwjgl_opengles_EGL_EGL_READ 12378L
#undef org_lwjgl_opengles_EGL_EGL_CORE_NATIVE_ENGINE
#define org_lwjgl_opengles_EGL_EGL_CORE_NATIVE_ENGINE 12379L
/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    eglGetError
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengles_EGL_eglGetError
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglGetDisplay
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGL_neglGetDisplay
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglInitialize
 * Signature: (JJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglInitialize
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglTerminate
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglTerminate
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglQueryString
 * Signature: (JI)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_lwjgl_opengles_EGL_neglQueryString
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglGetConfigs
 * Signature: (JJIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglGetConfigs
  (JNIEnv *, jclass, jlong, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglChooseConfig
 * Signature: (JJJIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglChooseConfig
  (JNIEnv *, jclass, jlong, jlong, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglGetConfigAttrib
 * Signature: (JJIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglGetConfigAttrib
  (JNIEnv *, jclass, jlong, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglCreateWindowSurface
 * Signature: (JJJJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGL_neglCreateWindowSurface
  (JNIEnv *, jclass, jlong, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglCreatePbufferSurface
 * Signature: (JJJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGL_neglCreatePbufferSurface
  (JNIEnv *, jclass, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglSurfaceAttrib
 * Signature: (JJII)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglSurfaceAttrib
  (JNIEnv *, jclass, jlong, jlong, jint, jint);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglDestroySurface
 * Signature: (JJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglDestroySurface
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglQuerySurface
 * Signature: (JJIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglQuerySurface
  (JNIEnv *, jclass, jlong, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    eglBindAPI
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_eglBindAPI
  (JNIEnv *, jclass, jint);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    eglQueryAPI
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengles_EGL_eglQueryAPI
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    eglReleaseThread
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_eglReleaseThread
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglSwapInterval
 * Signature: (JI)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglSwapInterval
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglCreateContext
 * Signature: (JJJJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGL_neglCreateContext
  (JNIEnv *, jclass, jlong, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglDestroyContext
 * Signature: (JJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglDestroyContext
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglMakeCurrent
 * Signature: (JJJJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglMakeCurrent
  (JNIEnv *, jclass, jlong, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglGetCurrentContext
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGL_neglGetCurrentContext
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglGetCurrentSurface
 * Signature: (I)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGL_neglGetCurrentSurface
  (JNIEnv *, jclass, jint);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglGetCurrentDisplay
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengles_EGL_neglGetCurrentDisplay
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglQueryContext
 * Signature: (JJIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglQueryContext
  (JNIEnv *, jclass, jlong, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    eglWaitClient
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_eglWaitClient
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    eglWaitGL
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_eglWaitGL
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    eglWaitNative
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_eglWaitNative
  (JNIEnv *, jclass, jint);

/*
 * Class:     org_lwjgl_opengles_EGL
 * Method:    neglSwapBuffers
 * Signature: (JJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGL_neglSwapBuffers
  (JNIEnv *, jclass, jlong, jlong);

#ifdef __cplusplus
}
#endif
#endif
