/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_WindowsDisplay */

#ifndef _Included_org_lwjgl_opengl_WindowsDisplay
#define _Included_org_lwjgl_opengl_WindowsDisplay
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengl_WindowsDisplay_GAMMA_LENGTH
#define org_lwjgl_opengl_WindowsDisplay_GAMMA_LENGTH 256L
#undef org_lwjgl_opengl_WindowsDisplay_WM_WINDOWPOSCHANGED
#define org_lwjgl_opengl_WindowsDisplay_WM_WINDOWPOSCHANGED 71L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MOVE
#define org_lwjgl_opengl_WindowsDisplay_WM_MOVE 3L
#undef org_lwjgl_opengl_WindowsDisplay_WM_CANCELMODE
#define org_lwjgl_opengl_WindowsDisplay_WM_CANCELMODE 31L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MOUSEMOVE
#define org_lwjgl_opengl_WindowsDisplay_WM_MOUSEMOVE 512L
#undef org_lwjgl_opengl_WindowsDisplay_WM_LBUTTONDOWN
#define org_lwjgl_opengl_WindowsDisplay_WM_LBUTTONDOWN 513L
#undef org_lwjgl_opengl_WindowsDisplay_WM_LBUTTONUP
#define org_lwjgl_opengl_WindowsDisplay_WM_LBUTTONUP 514L
#undef org_lwjgl_opengl_WindowsDisplay_WM_LBUTTONDBLCLK
#define org_lwjgl_opengl_WindowsDisplay_WM_LBUTTONDBLCLK 515L
#undef org_lwjgl_opengl_WindowsDisplay_WM_RBUTTONDOWN
#define org_lwjgl_opengl_WindowsDisplay_WM_RBUTTONDOWN 516L
#undef org_lwjgl_opengl_WindowsDisplay_WM_RBUTTONUP
#define org_lwjgl_opengl_WindowsDisplay_WM_RBUTTONUP 517L
#undef org_lwjgl_opengl_WindowsDisplay_WM_RBUTTONDBLCLK
#define org_lwjgl_opengl_WindowsDisplay_WM_RBUTTONDBLCLK 518L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MBUTTONDOWN
#define org_lwjgl_opengl_WindowsDisplay_WM_MBUTTONDOWN 519L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MBUTTONUP
#define org_lwjgl_opengl_WindowsDisplay_WM_MBUTTONUP 520L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MBUTTONDBLCLK
#define org_lwjgl_opengl_WindowsDisplay_WM_MBUTTONDBLCLK 521L
#undef org_lwjgl_opengl_WindowsDisplay_WM_XBUTTONDOWN
#define org_lwjgl_opengl_WindowsDisplay_WM_XBUTTONDOWN 523L
#undef org_lwjgl_opengl_WindowsDisplay_WM_XBUTTONUP
#define org_lwjgl_opengl_WindowsDisplay_WM_XBUTTONUP 524L
#undef org_lwjgl_opengl_WindowsDisplay_WM_XBUTTONDBLCLK
#define org_lwjgl_opengl_WindowsDisplay_WM_XBUTTONDBLCLK 525L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MOUSEWHEEL
#define org_lwjgl_opengl_WindowsDisplay_WM_MOUSEWHEEL 522L
#undef org_lwjgl_opengl_WindowsDisplay_WM_CAPTURECHANGED
#define org_lwjgl_opengl_WindowsDisplay_WM_CAPTURECHANGED 533L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MOUSELEAVE
#define org_lwjgl_opengl_WindowsDisplay_WM_MOUSELEAVE 675L
#undef org_lwjgl_opengl_WindowsDisplay_WM_ENTERSIZEMOVE
#define org_lwjgl_opengl_WindowsDisplay_WM_ENTERSIZEMOVE 561L
#undef org_lwjgl_opengl_WindowsDisplay_WM_EXITSIZEMOVE
#define org_lwjgl_opengl_WindowsDisplay_WM_EXITSIZEMOVE 562L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SIZING
#define org_lwjgl_opengl_WindowsDisplay_WM_SIZING 532L
#undef org_lwjgl_opengl_WindowsDisplay_WM_KEYDOWN
#define org_lwjgl_opengl_WindowsDisplay_WM_KEYDOWN 256L
#undef org_lwjgl_opengl_WindowsDisplay_WM_KEYUP
#define org_lwjgl_opengl_WindowsDisplay_WM_KEYUP 257L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SYSKEYUP
#define org_lwjgl_opengl_WindowsDisplay_WM_SYSKEYUP 261L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SYSKEYDOWN
#define org_lwjgl_opengl_WindowsDisplay_WM_SYSKEYDOWN 260L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SYSCHAR
#define org_lwjgl_opengl_WindowsDisplay_WM_SYSCHAR 262L
#undef org_lwjgl_opengl_WindowsDisplay_WM_CHAR
#define org_lwjgl_opengl_WindowsDisplay_WM_CHAR 258L
#undef org_lwjgl_opengl_WindowsDisplay_WM_GETICON
#define org_lwjgl_opengl_WindowsDisplay_WM_GETICON 127L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SETICON
#define org_lwjgl_opengl_WindowsDisplay_WM_SETICON 128L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SETCURSOR
#define org_lwjgl_opengl_WindowsDisplay_WM_SETCURSOR 32L
#undef org_lwjgl_opengl_WindowsDisplay_WM_MOUSEACTIVATE
#define org_lwjgl_opengl_WindowsDisplay_WM_MOUSEACTIVATE 33L
#undef org_lwjgl_opengl_WindowsDisplay_WM_QUIT
#define org_lwjgl_opengl_WindowsDisplay_WM_QUIT 18L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SYSCOMMAND
#define org_lwjgl_opengl_WindowsDisplay_WM_SYSCOMMAND 274L
#undef org_lwjgl_opengl_WindowsDisplay_WM_PAINT
#define org_lwjgl_opengl_WindowsDisplay_WM_PAINT 15L
#undef org_lwjgl_opengl_WindowsDisplay_WM_KILLFOCUS
#define org_lwjgl_opengl_WindowsDisplay_WM_KILLFOCUS 8L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SETFOCUS
#define org_lwjgl_opengl_WindowsDisplay_WM_SETFOCUS 7L
#undef org_lwjgl_opengl_WindowsDisplay_SC_SIZE
#define org_lwjgl_opengl_WindowsDisplay_SC_SIZE 61440L
#undef org_lwjgl_opengl_WindowsDisplay_SC_MOVE
#define org_lwjgl_opengl_WindowsDisplay_SC_MOVE 61456L
#undef org_lwjgl_opengl_WindowsDisplay_SC_MINIMIZE
#define org_lwjgl_opengl_WindowsDisplay_SC_MINIMIZE 61472L
#undef org_lwjgl_opengl_WindowsDisplay_SC_MAXIMIZE
#define org_lwjgl_opengl_WindowsDisplay_SC_MAXIMIZE 61488L
#undef org_lwjgl_opengl_WindowsDisplay_SC_NEXTWINDOW
#define org_lwjgl_opengl_WindowsDisplay_SC_NEXTWINDOW 61504L
#undef org_lwjgl_opengl_WindowsDisplay_SC_PREVWINDOW
#define org_lwjgl_opengl_WindowsDisplay_SC_PREVWINDOW 61520L
#undef org_lwjgl_opengl_WindowsDisplay_SC_CLOSE
#define org_lwjgl_opengl_WindowsDisplay_SC_CLOSE 61536L
#undef org_lwjgl_opengl_WindowsDisplay_SC_VSCROLL
#define org_lwjgl_opengl_WindowsDisplay_SC_VSCROLL 61552L
#undef org_lwjgl_opengl_WindowsDisplay_SC_HSCROLL
#define org_lwjgl_opengl_WindowsDisplay_SC_HSCROLL 61568L
#undef org_lwjgl_opengl_WindowsDisplay_SC_MOUSEMENU
#define org_lwjgl_opengl_WindowsDisplay_SC_MOUSEMENU 61584L
#undef org_lwjgl_opengl_WindowsDisplay_SC_KEYMENU
#define org_lwjgl_opengl_WindowsDisplay_SC_KEYMENU 61696L
#undef org_lwjgl_opengl_WindowsDisplay_SC_ARRANGE
#define org_lwjgl_opengl_WindowsDisplay_SC_ARRANGE 61712L
#undef org_lwjgl_opengl_WindowsDisplay_SC_RESTORE
#define org_lwjgl_opengl_WindowsDisplay_SC_RESTORE 61728L
#undef org_lwjgl_opengl_WindowsDisplay_SC_TASKLIST
#define org_lwjgl_opengl_WindowsDisplay_SC_TASKLIST 61744L
#undef org_lwjgl_opengl_WindowsDisplay_SC_SCREENSAVE
#define org_lwjgl_opengl_WindowsDisplay_SC_SCREENSAVE 61760L
#undef org_lwjgl_opengl_WindowsDisplay_SC_HOTKEY
#define org_lwjgl_opengl_WindowsDisplay_SC_HOTKEY 61776L
#undef org_lwjgl_opengl_WindowsDisplay_SC_DEFAULT
#define org_lwjgl_opengl_WindowsDisplay_SC_DEFAULT 61792L
#undef org_lwjgl_opengl_WindowsDisplay_SC_MONITORPOWER
#define org_lwjgl_opengl_WindowsDisplay_SC_MONITORPOWER 61808L
#undef org_lwjgl_opengl_WindowsDisplay_SC_CONTEXTHELP
#define org_lwjgl_opengl_WindowsDisplay_SC_CONTEXTHELP 61824L
#undef org_lwjgl_opengl_WindowsDisplay_SC_SEPARATOR
#define org_lwjgl_opengl_WindowsDisplay_SC_SEPARATOR 61455L
#undef org_lwjgl_opengl_WindowsDisplay_SM_CXCURSOR
#define org_lwjgl_opengl_WindowsDisplay_SM_CXCURSOR 13L
#undef org_lwjgl_opengl_WindowsDisplay_SM_CYCURSOR
#define org_lwjgl_opengl_WindowsDisplay_SM_CYCURSOR 14L
#undef org_lwjgl_opengl_WindowsDisplay_SM_CMOUSEBUTTONS
#define org_lwjgl_opengl_WindowsDisplay_SM_CMOUSEBUTTONS 43L
#undef org_lwjgl_opengl_WindowsDisplay_SM_MOUSEWHEELPRESENT
#define org_lwjgl_opengl_WindowsDisplay_SM_MOUSEWHEELPRESENT 75L
#undef org_lwjgl_opengl_WindowsDisplay_SIZE_RESTORED
#define org_lwjgl_opengl_WindowsDisplay_SIZE_RESTORED 0L
#undef org_lwjgl_opengl_WindowsDisplay_SIZE_MINIMIZED
#define org_lwjgl_opengl_WindowsDisplay_SIZE_MINIMIZED 1L
#undef org_lwjgl_opengl_WindowsDisplay_SIZE_MAXIMIZED
#define org_lwjgl_opengl_WindowsDisplay_SIZE_MAXIMIZED 2L
#undef org_lwjgl_opengl_WindowsDisplay_WM_SIZE
#define org_lwjgl_opengl_WindowsDisplay_WM_SIZE 5L
#undef org_lwjgl_opengl_WindowsDisplay_WM_ACTIVATE
#define org_lwjgl_opengl_WindowsDisplay_WM_ACTIVATE 6L
#undef org_lwjgl_opengl_WindowsDisplay_WA_INACTIVE
#define org_lwjgl_opengl_WindowsDisplay_WA_INACTIVE 0L
#undef org_lwjgl_opengl_WindowsDisplay_WA_ACTIVE
#define org_lwjgl_opengl_WindowsDisplay_WA_ACTIVE 1L
#undef org_lwjgl_opengl_WindowsDisplay_WA_CLICKACTIVE
#define org_lwjgl_opengl_WindowsDisplay_WA_CLICKACTIVE 2L
#undef org_lwjgl_opengl_WindowsDisplay_SW_NORMAL
#define org_lwjgl_opengl_WindowsDisplay_SW_NORMAL 1L
#undef org_lwjgl_opengl_WindowsDisplay_SW_SHOWMINNOACTIVE
#define org_lwjgl_opengl_WindowsDisplay_SW_SHOWMINNOACTIVE 7L
#undef org_lwjgl_opengl_WindowsDisplay_SW_SHOWDEFAULT
#define org_lwjgl_opengl_WindowsDisplay_SW_SHOWDEFAULT 10L
#undef org_lwjgl_opengl_WindowsDisplay_SW_RESTORE
#define org_lwjgl_opengl_WindowsDisplay_SW_RESTORE 9L
#undef org_lwjgl_opengl_WindowsDisplay_SW_MAXIMIZE
#define org_lwjgl_opengl_WindowsDisplay_SW_MAXIMIZE 3L
#undef org_lwjgl_opengl_WindowsDisplay_ICON_SMALL
#define org_lwjgl_opengl_WindowsDisplay_ICON_SMALL 0L
#undef org_lwjgl_opengl_WindowsDisplay_ICON_BIG
#define org_lwjgl_opengl_WindowsDisplay_ICON_BIG 1L
#undef org_lwjgl_opengl_WindowsDisplay_HWND_TOP
#define org_lwjgl_opengl_WindowsDisplay_HWND_TOP 0i64
#undef org_lwjgl_opengl_WindowsDisplay_HWND_BOTTOM
#define org_lwjgl_opengl_WindowsDisplay_HWND_BOTTOM 1i64
#undef org_lwjgl_opengl_WindowsDisplay_HWND_TOPMOST
#define org_lwjgl_opengl_WindowsDisplay_HWND_TOPMOST -1i64
#undef org_lwjgl_opengl_WindowsDisplay_HWND_NOTOPMOST
#define org_lwjgl_opengl_WindowsDisplay_HWND_NOTOPMOST -2i64
#undef org_lwjgl_opengl_WindowsDisplay_SWP_NOSIZE
#define org_lwjgl_opengl_WindowsDisplay_SWP_NOSIZE 1L
#undef org_lwjgl_opengl_WindowsDisplay_SWP_NOMOVE
#define org_lwjgl_opengl_WindowsDisplay_SWP_NOMOVE 2L
#undef org_lwjgl_opengl_WindowsDisplay_SWP_NOZORDER
#define org_lwjgl_opengl_WindowsDisplay_SWP_NOZORDER 4L
#undef org_lwjgl_opengl_WindowsDisplay_SWP_FRAMECHANGED
#define org_lwjgl_opengl_WindowsDisplay_SWP_FRAMECHANGED 32L
#undef org_lwjgl_opengl_WindowsDisplay_GWL_STYLE
#define org_lwjgl_opengl_WindowsDisplay_GWL_STYLE -16L
#undef org_lwjgl_opengl_WindowsDisplay_GWL_EXSTYLE
#define org_lwjgl_opengl_WindowsDisplay_GWL_EXSTYLE -20L
#undef org_lwjgl_opengl_WindowsDisplay_WS_THICKFRAME
#define org_lwjgl_opengl_WindowsDisplay_WS_THICKFRAME 262144L
#undef org_lwjgl_opengl_WindowsDisplay_WS_MAXIMIZEBOX
#define org_lwjgl_opengl_WindowsDisplay_WS_MAXIMIZEBOX 65536L
#undef org_lwjgl_opengl_WindowsDisplay_HTCLIENT
#define org_lwjgl_opengl_WindowsDisplay_HTCLIENT 1L
#undef org_lwjgl_opengl_WindowsDisplay_MK_XBUTTON1
#define org_lwjgl_opengl_WindowsDisplay_MK_XBUTTON1 32L
#undef org_lwjgl_opengl_WindowsDisplay_MK_XBUTTON2
#define org_lwjgl_opengl_WindowsDisplay_MK_XBUTTON2 64L
#undef org_lwjgl_opengl_WindowsDisplay_XBUTTON1
#define org_lwjgl_opengl_WindowsDisplay_XBUTTON1 1L
#undef org_lwjgl_opengl_WindowsDisplay_XBUTTON2
#define org_lwjgl_opengl_WindowsDisplay_XBUTTON2 2L
/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nCreateWindow
 * Signature: (IIIIZZJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nCreateWindow
  (JNIEnv *, jclass, jint, jint, jint, jint, jboolean, jboolean, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nReleaseDC
 * Signature: (JJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nReleaseDC
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nDestroyWindow
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nDestroyWindow
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    clipCursor
 * Signature: (Ljava/nio/IntBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_clipCursor
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nSwitchDisplayMode
 * Signature: (Lorg/lwjgl/opengl/DisplayMode;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nSwitchDisplayMode
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    showWindow
 * Signature: (JI)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_showWindow
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    setForegroundWindow
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_setForegroundWindow
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    setFocus
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_setFocus
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nResetDisplayMode
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nResetDisplayMode
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    convertToNativeRamp
 * Signature: (Ljava/nio/FloatBuffer;)Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_WindowsDisplay_convertToNativeRamp
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getCurrentGammaRamp
 * Signature: ()Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getCurrentGammaRamp
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nSetGammaRamp
 * Signature: (Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nSetGammaRamp
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nGetVersion
 * Signature: (Ljava/lang/String;)Lorg/lwjgl/opengl/WindowsFileVersion;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nGetVersion
  (JNIEnv *, jobject, jstring);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getCurrentDisplayMode
 * Signature: ()Lorg/lwjgl/opengl/DisplayMode;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getCurrentDisplayMode
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nSetTitle
 * Signature: (JJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nSetTitle
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nUpdate
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nUpdate
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nReshape
 * Signature: (JIIIIZZ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nReshape
  (JNIEnv *, jclass, jlong, jint, jint, jint, jint, jboolean, jboolean);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getAvailableDisplayModes
 * Signature: ()[Lorg/lwjgl/opengl/DisplayMode;
 */
JNIEXPORT jobjectArray JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getAvailableDisplayModes
  (JNIEnv *, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nSetCursorPosition
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nSetCursorPosition
  (JNIEnv *, jclass, jint, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nSetNativeCursor
 * Signature: (JLjava/lang/Object;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nSetNativeCursor
  (JNIEnv *, jclass, jlong, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getSystemMetrics
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getSystemMetrics
  (JNIEnv *, jclass, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getDllInstance
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getDllInstance
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getDC
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getDC
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getDesktopWindow
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getDesktopWindow
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getForegroundWindow
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getForegroundWindow
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nCreateCursor
 * Signature: (IIIIILjava/nio/IntBuffer;ILjava/nio/IntBuffer;I)Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nCreateCursor
  (JNIEnv *, jclass, jint, jint, jint, jint, jint, jobject, jint, jobject, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    doDestroyCursor
 * Signature: (Ljava/lang/Object;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_doDestroyCursor
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nGetPbufferCapabilities
 * Signature: (Lorg/lwjgl/opengl/PixelFormat;)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nGetPbufferCapabilities
  (JNIEnv *, jobject, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    createIcon
 * Signature: (IILjava/nio/IntBuffer;)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_createIcon
  (JNIEnv *, jclass, jint, jint, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    destroyIcon
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_destroyIcon
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    sendMessage
 * Signature: (JJJJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_sendMessage
  (JNIEnv *, jclass, jlong, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    setWindowLongPtr
 * Signature: (JIJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_setWindowLongPtr
  (JNIEnv *, jclass, jlong, jint, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getWindowLongPtr
 * Signature: (JI)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getWindowLongPtr
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    setWindowPos
 * Signature: (JJIIIIJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_WindowsDisplay_setWindowPos
  (JNIEnv *, jclass, jlong, jlong, jint, jint, jint, jint, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nSetCapture
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nSetCapture
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nReleaseCapture
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nReleaseCapture
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getClientRect
 * Signature: (JLjava/nio/IntBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getClientRect
  (JNIEnv *, jclass, jlong, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    clientToScreen
 * Signature: (JLjava/nio/IntBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_clientToScreen
  (JNIEnv *, jclass, jlong, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    setWindowProc
 * Signature: (Ljava/lang/reflect/Method;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_WindowsDisplay_setWindowProc
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    defWindowProc
 * Signature: (JIJJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_WindowsDisplay_defWindowProc
  (JNIEnv *, jclass, jlong, jint, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    getWindowRect
 * Signature: (JLjava/nio/IntBuffer;)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_WindowsDisplay_getWindowRect
  (JNIEnv *, jobject, jlong, jobject);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    nTrackMouseEvent
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_WindowsDisplay_nTrackMouseEvent
  (JNIEnv *, jobject, jlong);

/*
 * Class:     org_lwjgl_opengl_WindowsDisplay
 * Method:    adjustWindowRectEx
 * Signature: (Ljava/nio/IntBuffer;IZI)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_WindowsDisplay_adjustWindowRectEx
  (JNIEnv *, jobject, jobject, jint, jboolean, jint);

#ifdef __cplusplus
}
#endif
#endif
