/*
 * Copyright (c) 2002-2011 LWJGL Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'LWJGL' nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lwjgl.test.mapped;

import org.lwjgl.util.mapped.MappedObjectClassLoader;
import org.lwjgl.util.mapped.MappedObjectTransformer;

/** @author Riven */
@SuppressWarnings("static-access")
public class TestMappedObject {

	static {
		boolean assertsEnabled = false;
		assert assertsEnabled = true; // Intentional side effect!!!
		if ( !assertsEnabled )
			throw new RuntimeException("Asserts must be enabled for this test.");
	}

	public static void main(String[] args) throws Exception {
		MappedObjectTransformer.register(MappedFloat.class);
		MappedObjectTransformer.register(MappedVec2.class);
		MappedObjectTransformer.register(MappedVec3.class);
		MappedObjectTransformer.register(MappedSomething.class);
		MappedObjectTransformer.register(MappedObjectTests3.Xyz.class);
		MappedObjectTransformer.register(MappedObjectTests4.MappedPointer.class);
		MappedObjectTransformer.register(MappedObjectTests4.MappedCacheLinePadded.class);
		MappedObjectTransformer.register(MappedObjectTests4.MappedFieldCacheLinePadded.class);

		if ( MappedObjectClassLoader.fork(TestMappedObject.class, args) ) {
			return;
		}

		MappedObjectTests1.testViewField();

		MappedObjectTests2.testFields();

		// MappedObjectBench.benchmarkMapped();
		// MappedObjectBench.benchmarkInstances();
		// MappedObjectBench.benchmarkIndirectArray();
		// MappedObjectBench.benchmarkDirectArray();
		// MappedObjectBench.benchmarkUnsafe();

		MappedObjectTests3.testMappedBuffer();
		MappedObjectTests3.testForeach();
		MappedObjectTests3.testConstructor();
		MappedObjectTests3.testMappedSet();

		MappedObjectTests4.testLocalView();
		//MappedObjectTests4.testLWJGL();
		MappedObjectTests4.testPointer();
		MappedObjectTests4.testCacheLineAlignment();
		MappedObjectTests4.testCacheLinePadding();
		MappedObjectTests4.testCacheLinePaddingPOJO();

		System.out.println("done");
	}

}