/*
 * Copyright (c) 2002-2008 LWJGL Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'LWJGL' nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lwjgl.opengl;

public interface NV_texture_shader2 {
	int GL_DOT_PRODUCT_TEXTURE_3D_NV = 0x86EF;
	int GL_HILO_NV = 0x86F4;
	int GL_DSDT_NV = 0x86F5;
	int GL_DSDT_MAG_NV = 0x86F6;
	int GL_DSDT_MAG_VIB_NV = 0x86F7;
	int GL_UNSIGNED_INT_S8_S8_8_8_NV = 0x86DA;
	int GL_UNSIGNED_INT_8_8_S8_S8_REV_NV = 0x86DB;
	int GL_SIGNED_RGBA_NV = 0x86FB;
	int GL_SIGNED_RGBA8_NV = 0x86FC;
	int GL_SIGNED_RGB_NV = 0x86FE;
	int GL_SIGNED_RGB8_NV = 0x86FF;
	int GL_SIGNED_LUMINANCE_NV = 0x8701;
	int GL_SIGNED_LUMINANCE8_NV = 0x8702;
	int GL_SIGNED_LUMINANCE_ALPHA_NV = 0x8703;
	int GL_SIGNED_LUMINANCE8_ALPHA8_NV = 0x8704;
	int GL_SIGNED_ALPHA_NV = 0x8705;
	int GL_SIGNED_ALPHA8_NV = 0x8706;
	int GL_SIGNED_INTENSITY_NV = 0x8707;
	int GL_SIGNED_INTENSITY8_NV = 0x8708;
	int GL_SIGNED_RGB_UNSIGNED_ALPHA_NV = 0x870C;
	int GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV = 0x870D;
	int GL_HILO16_NV = 0x86F8;
	int GL_SIGNED_HILO_NV = 0x86F9;
	int GL_SIGNED_HILO16_NV = 0x86FA;
	int GL_DSDT8_NV = 0x8709;
	int GL_DSDT8_MAG8_NV = 0x870A;
	int GL_DSDT_MAG_INTENSITY_NV = 0x86DC;
	int GL_DSDT8_MAG8_INTENSITY8_NV = 0x870B;
}
