/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_MacOSXNativeKeyboard */

#ifndef _Included_org_lwjgl_opengl_MacOSXNativeKeyboard
#define _Included_org_lwjgl_opengl_MacOSXNativeKeyboard
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengl_MacOSXNativeKeyboard_QUEUE_SIZE
#define org_lwjgl_opengl_MacOSXNativeKeyboard_QUEUE_SIZE 200L
/*
 * Class:     org_lwjgl_opengl_MacOSXNativeKeyboard
 * Method:    nRegisterKeyListener
 * Signature: (Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXNativeKeyboard_nRegisterKeyListener
  (JNIEnv *, jobject, jobject);

/*
 * Class:     org_lwjgl_opengl_MacOSXNativeKeyboard
 * Method:    nUnregisterKeyListener
 * Signature: (Ljava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXNativeKeyboard_nUnregisterKeyListener
  (JNIEnv *, jobject, jobject);

#ifdef __cplusplus
}
#endif
#endif
