/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_input_Mouse */

#ifndef _Included_org_lwjgl_input_Mouse
#define _Included_org_lwjgl_input_Mouse
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_input_Mouse_EVENT_SIZE
#define org_lwjgl_input_Mouse_EVENT_SIZE 22L
#undef org_lwjgl_input_Mouse_BUFFER_SIZE
#define org_lwjgl_input_Mouse_BUFFER_SIZE 50L
#ifdef __cplusplus
}
#endif
#endif
