/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_LinuxPeerInfo */

#ifndef _Included_org_lwjgl_opengl_LinuxPeerInfo
#define _Included_org_lwjgl_opengl_LinuxPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_LinuxPeerInfo
 * Method:    createHandle
 * Signature: ()Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_LinuxPeerInfo_createHandle
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_LinuxPeerInfo
 * Method:    nGetDisplay
 * Signature: (Ljava/nio/ByteBuffer;)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxPeerInfo_nGetDisplay
  (JNIEnv *, jclass, jobject);

/*
 * Class:     org_lwjgl_opengl_LinuxPeerInfo
 * Method:    nGetDrawable
 * Signature: (Ljava/nio/ByteBuffer;)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxPeerInfo_nGetDrawable
  (JNIEnv *, jclass, jobject);

#ifdef __cplusplus
}
#endif
#endif
