/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengles_EGLKHRReusableSync */

#ifndef _Included_org_lwjgl_opengles_EGLKHRReusableSync
#define _Included_org_lwjgl_opengles_EGLKHRReusableSync
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_REUSABLE_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_REUSABLE_KHR 12538L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_TYPE_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_TYPE_KHR 12535L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_STATUS_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_STATUS_KHR 12529L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_SIGNALED_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_SIGNALED_KHR 12530L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_UNSIGNALED_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_UNSIGNALED_KHR 12531L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_FLUSH_COMMANDS_BIT_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_SYNC_FLUSH_COMMANDS_BIT_KHR 1L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_FOREVER_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_FOREVER_KHR -1i64
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_TIMEOUT_EXPIRED_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_TIMEOUT_EXPIRED_KHR 12533L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_CONDITION_SATISFIED_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_CONDITION_SATISFIED_KHR 12534L
#undef org_lwjgl_opengles_EGLKHRReusableSync_EGL_NO_SYNC_KHR
#define org_lwjgl_opengles_EGLKHRReusableSync_EGL_NO_SYNC_KHR 0i64
/*
 * Class:     org_lwjgl_opengles_EGLKHRReusableSync
 * Method:    initNativeStubs
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengles_EGLKHRReusableSync_initNativeStubs
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengles_EGLKHRReusableSync
 * Method:    neglSignalSyncKHR
 * Signature: (JJI)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengles_EGLKHRReusableSync_neglSignalSyncKHR
  (JNIEnv *, jclass, jlong, jlong, jint);

#ifdef __cplusplus
}
#endif
#endif
