/* 
 * Copyright (c) 2002-2008 LWJGL Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'LWJGL' nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lwjgl.util.glu;

import static org.lwjgl.util.glu.GLU.*;

/**
 * Registry.java
 *
 *
 * Created 11-jan-2004
 * @author Erik Duijs
 */
public class Registry extends Util {

	private static final String versionString = "1.3";
	private static final String extensionString =
		"GLU_EXT_nurbs_tessellator " + "GLU_EXT_object_space_tess ";

	/**
	 * Method gluGetString
	 * @param name
	 * @return String
	 */
	public static String gluGetString(int name) {

		if (name == GLU_VERSION) {
			return versionString;
		} else if (name == GLU_EXTENSIONS) {
			return extensionString;
		}
		return null;
	}

	/**
	 * Method gluCheckExtension
	 *
	 * @param extName is an extension name.
	 * @param extString is a string of extensions separated by blank(s). There may or
	 * may not be leading or trailing blank(s) in extString.
	 * This works in cases of extensions being prefixes of another like
	 * GL_EXT_texture and GL_EXT_texture3D.
	 * @return boolean true if extName is found otherwise it returns false.
	 */
	public static boolean gluCheckExtension(String extName, String extString) {
		if (extString == null || extName == null)
			return false;

		return extString.indexOf(extName) != -1;
	}
}
