/*
 * Copyright (c) 2002-2010 LWJGL Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'LWJGL' nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lwjgl.opencl;

import org.lwjgl.PointerWrapperAbstract;

/**
 * Instances of this class can be used to handle OpenCL event callbacks. A single
 * CLEventCallback instance should only be used on events generated from the same
 * CLCommandQueue or on user events associated with the same CLContext.
 *
 * @author Spasi
 */
public abstract class CLEventCallback extends PointerWrapperAbstract {

	private CLObjectRegistry<CLEvent> eventRegistry;

	protected CLEventCallback() {
		super(CallbackUtil.getEventCallback());
	}

	/**
	 * Sets the eventRegistry that contains the CLEvents to which we're registered.
	 *
	 * @param eventRegistry the CLEvent object registry
	 */
	void setRegistry(final CLObjectRegistry<CLEvent> eventRegistry) {
		this.eventRegistry = eventRegistry;
	}

	/**
	 * Called from native code.
	 *
	 * @param event_address the CLEvent object pointer
	 */
	private void handleMessage(long event_address, int event_command_exec_status) {
		handleMessage(eventRegistry.getObject(event_address), event_command_exec_status);
	}

	/**
	 * The callback method.
	 *
	 * @param event                     the CLEvent object
	 * @param event_command_exec_status the execution status
	 */
	protected abstract void handleMessage(CLEvent event, int event_command_exec_status);

}