/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_LinuxMouse */

#ifndef _Included_org_lwjgl_opengl_LinuxMouse
#define _Included_org_lwjgl_opengl_LinuxMouse
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengl_LinuxMouse_POINTER_WARP_BORDER
#define org_lwjgl_opengl_LinuxMouse_POINTER_WARP_BORDER 10L
#undef org_lwjgl_opengl_LinuxMouse_WHEEL_SCALE
#define org_lwjgl_opengl_LinuxMouse_WHEEL_SCALE 120L
#undef org_lwjgl_opengl_LinuxMouse_Button1
#define org_lwjgl_opengl_LinuxMouse_Button1 1L
#undef org_lwjgl_opengl_LinuxMouse_Button2
#define org_lwjgl_opengl_LinuxMouse_Button2 2L
#undef org_lwjgl_opengl_LinuxMouse_Button3
#define org_lwjgl_opengl_LinuxMouse_Button3 3L
#undef org_lwjgl_opengl_LinuxMouse_Button4
#define org_lwjgl_opengl_LinuxMouse_Button4 4L
#undef org_lwjgl_opengl_LinuxMouse_Button5
#define org_lwjgl_opengl_LinuxMouse_Button5 5L
#undef org_lwjgl_opengl_LinuxMouse_Button6
#define org_lwjgl_opengl_LinuxMouse_Button6 6L
#undef org_lwjgl_opengl_LinuxMouse_Button7
#define org_lwjgl_opengl_LinuxMouse_Button7 7L
#undef org_lwjgl_opengl_LinuxMouse_Button8
#define org_lwjgl_opengl_LinuxMouse_Button8 8L
#undef org_lwjgl_opengl_LinuxMouse_Button9
#define org_lwjgl_opengl_LinuxMouse_Button9 9L
#undef org_lwjgl_opengl_LinuxMouse_ButtonPress
#define org_lwjgl_opengl_LinuxMouse_ButtonPress 4L
#undef org_lwjgl_opengl_LinuxMouse_ButtonRelease
#define org_lwjgl_opengl_LinuxMouse_ButtonRelease 5L
/*
 * Class:     org_lwjgl_opengl_LinuxMouse
 * Method:    nSendWarpEvent
 * Signature: (JJJII)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxMouse_nSendWarpEvent
  (JNIEnv *, jclass, jlong, jlong, jlong, jint, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxMouse
 * Method:    nGetWindowHeight
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxMouse_nGetWindowHeight
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxMouse
 * Method:    nGetWindowWidth
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxMouse_nGetWindowWidth
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxMouse
 * Method:    nGetButtonCount
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxMouse_nGetButtonCount
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxMouse
 * Method:    nQueryPointer
 * Signature: (JJLjava/nio/IntBuffer;)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxMouse_nQueryPointer
  (JNIEnv *, jclass, jlong, jlong, jobject);

/*
 * Class:     org_lwjgl_opengl_LinuxMouse
 * Method:    nWarpCursor
 * Signature: (JJII)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxMouse_nWarpCursor
  (JNIEnv *, jclass, jlong, jlong, jint, jint);

#ifdef __cplusplus
}
#endif
#endif
