/*
 * Copyright (c) 2002-2008 LWJGL Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'LWJGL' nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lwjgl.opengl;

import org.lwjgl.util.generator.*;
import org.lwjgl.util.generator.opengl.*;

import java.nio.Buffer;
import java.nio.IntBuffer;

public interface APPLE_element_array {

	/**
	 * Accepted by the &lt;array&gt; parameter of EnableClientState and
	 * DisableClientState and the &lt;value&gt; parameter of IsEnabled:
	 */
	int GL_ELEMENT_ARRAY_APPLE = 0x8768;

	/**
	 * Accepted by the &lt;value&gt; parameter of GetBooleanv, GetIntegerv,
	 * GetFloatv, and GetDoublev:
	 */
	int GL_ELEMENT_ARRAY_TYPE_APPLE = 0x8769;

	/** Accepted by the &lt;pname&gt; parameter of GetPointerv: */
	int GL_ELEMENT_ARRAY_POINTER_APPLE = 0x876A;

	void glElementPointerAPPLE(@AutoType("pointer") @GLenum int type,
	                           @Check
	                           @Const
	                           @GLubyte
	                           @GLushort
	                           @GLuint Buffer pointer);

	void glDrawElementArrayAPPLE(@GLenum int mode, int first, @GLsizei int count);

	void glDrawRangeElementArrayAPPLE(@GLenum int mode, @GLuint int start, @GLuint int end, int first, @GLsizei int count);

	void glMultiDrawElementArrayAPPLE(@GLenum int mode,
	                                  @Const IntBuffer first,
	                                  @Const @Check("first.remaining()") @GLsizei IntBuffer count,
	                                  @AutoSize("first") @GLsizei int primcount);

	void glMultiDrawRangeElementArrayAPPLE(@GLenum int mode, @GLuint int start, @GLuint int end,
	                                       @Const IntBuffer first,
	                                       @Const @Check("first.remaining()") @GLsizei IntBuffer count,
	                                       @AutoSize("first") @GLsizei int primcount);

}