/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_LinuxDisplay */

#ifndef _Included_org_lwjgl_opengl_LinuxDisplay
#define _Included_org_lwjgl_opengl_LinuxDisplay
#ifdef __cplusplus
extern "C" {
#endif
#undef org_lwjgl_opengl_LinuxDisplay_CurrentTime
#define org_lwjgl_opengl_LinuxDisplay_CurrentTime 0L
#undef org_lwjgl_opengl_LinuxDisplay_GrabSuccess
#define org_lwjgl_opengl_LinuxDisplay_GrabSuccess 0L
#undef org_lwjgl_opengl_LinuxDisplay_AutoRepeatModeOff
#define org_lwjgl_opengl_LinuxDisplay_AutoRepeatModeOff 0L
#undef org_lwjgl_opengl_LinuxDisplay_AutoRepeatModeOn
#define org_lwjgl_opengl_LinuxDisplay_AutoRepeatModeOn 1L
#undef org_lwjgl_opengl_LinuxDisplay_AutoRepeatModeDefault
#define org_lwjgl_opengl_LinuxDisplay_AutoRepeatModeDefault 2L
#undef org_lwjgl_opengl_LinuxDisplay_None
#define org_lwjgl_opengl_LinuxDisplay_None 0L
#undef org_lwjgl_opengl_LinuxDisplay_KeyPressMask
#define org_lwjgl_opengl_LinuxDisplay_KeyPressMask 1L
#undef org_lwjgl_opengl_LinuxDisplay_KeyReleaseMask
#define org_lwjgl_opengl_LinuxDisplay_KeyReleaseMask 2L
#undef org_lwjgl_opengl_LinuxDisplay_ButtonPressMask
#define org_lwjgl_opengl_LinuxDisplay_ButtonPressMask 4L
#undef org_lwjgl_opengl_LinuxDisplay_ButtonReleaseMask
#define org_lwjgl_opengl_LinuxDisplay_ButtonReleaseMask 8L
#undef org_lwjgl_opengl_LinuxDisplay_NotifyAncestor
#define org_lwjgl_opengl_LinuxDisplay_NotifyAncestor 0L
#undef org_lwjgl_opengl_LinuxDisplay_NotifyNonlinear
#define org_lwjgl_opengl_LinuxDisplay_NotifyNonlinear 3L
#undef org_lwjgl_opengl_LinuxDisplay_NotifyPointer
#define org_lwjgl_opengl_LinuxDisplay_NotifyPointer 5L
#undef org_lwjgl_opengl_LinuxDisplay_NotifyPointerRoot
#define org_lwjgl_opengl_LinuxDisplay_NotifyPointerRoot 6L
#undef org_lwjgl_opengl_LinuxDisplay_NotifyDetailNone
#define org_lwjgl_opengl_LinuxDisplay_NotifyDetailNone 7L
#undef org_lwjgl_opengl_LinuxDisplay_SetModeInsert
#define org_lwjgl_opengl_LinuxDisplay_SetModeInsert 0L
#undef org_lwjgl_opengl_LinuxDisplay_SaveSetRoot
#define org_lwjgl_opengl_LinuxDisplay_SaveSetRoot 1L
#undef org_lwjgl_opengl_LinuxDisplay_SaveSetUnmap
#define org_lwjgl_opengl_LinuxDisplay_SaveSetUnmap 1L
#undef org_lwjgl_opengl_LinuxDisplay_X_SetInputFocus
#define org_lwjgl_opengl_LinuxDisplay_X_SetInputFocus 42L
#undef org_lwjgl_opengl_LinuxDisplay_FULLSCREEN_LEGACY
#define org_lwjgl_opengl_LinuxDisplay_FULLSCREEN_LEGACY 1L
#undef org_lwjgl_opengl_LinuxDisplay_FULLSCREEN_NETWM
#define org_lwjgl_opengl_LinuxDisplay_FULLSCREEN_NETWM 2L
#undef org_lwjgl_opengl_LinuxDisplay_WINDOWED
#define org_lwjgl_opengl_LinuxDisplay_WINDOWED 3L
#undef org_lwjgl_opengl_LinuxDisplay_XRANDR
#define org_lwjgl_opengl_LinuxDisplay_XRANDR 10L
#undef org_lwjgl_opengl_LinuxDisplay_XF86VIDMODE
#define org_lwjgl_opengl_LinuxDisplay_XF86VIDMODE 11L
#undef org_lwjgl_opengl_LinuxDisplay_NONE
#define org_lwjgl_opengl_LinuxDisplay_NONE 12L
/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetCurrentGammaRamp
 * Signature: (JI)Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetCurrentGammaRamp
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nIsXrandrSupported
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nIsXrandrSupported
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nIsXF86VidModeSupported
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nIsXF86VidModeSupported
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nIsNetWMFullscreenSupported
 * Signature: (JI)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nIsNetWMFullscreenSupported
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nLockAWT
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nLockAWT
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nUnlockAWT
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nUnlockAWT
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    callErrorHandler
 * Signature: (JJJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_callErrorHandler
  (JNIEnv *, jclass, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    setErrorHandler
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_setErrorHandler
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    resetErrorHandler
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_resetErrorHandler
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    synchronize
 * Signature: (JZ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_synchronize
  (JNIEnv *, jclass, jlong, jboolean);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    getErrorText
 * Signature: (JJ)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_lwjgl_opengl_LinuxDisplay_getErrorText
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    openDisplay
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_openDisplay
  (JNIEnv *, jclass);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    closeDisplay
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_closeDisplay
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetDefaultScreen
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetDefaultScreen
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nUngrabKeyboard
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nUngrabKeyboard
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGrabKeyboard
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGrabKeyboard
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGrabPointer
 * Signature: (JJJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGrabPointer
  (JNIEnv *, jclass, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSetViewPort
 * Signature: (JJI)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSetViewPort
  (JNIEnv *, jclass, jlong, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nUngrabPointer
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nUngrabPointer
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nDefineCursor
 * Signature: (JJJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nDefineCursor
  (JNIEnv *, jclass, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nCreateWindow
 * Signature: (JILjava/nio/ByteBuffer;Lorg/lwjgl/opengl/DisplayMode;IIIZJZ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nCreateWindow
  (JNIEnv *, jclass, jlong, jint, jobject, jobject, jint, jint, jint, jboolean, jlong, jboolean);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    getRootWindow
 * Signature: (JI)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_getRootWindow
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    hasProperty
 * Signature: (JJJ)Z
 */
JNIEXPORT jboolean JNICALL Java_org_lwjgl_opengl_LinuxDisplay_hasProperty
  (JNIEnv *, jclass, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    getParentWindow
 * Signature: (JJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_getParentWindow
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    getChildCount
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_getChildCount
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    mapRaised
 * Signature: (JJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_mapRaised
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    reparentWindow
 * Signature: (JJJII)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_reparentWindow
  (JNIEnv *, jclass, jlong, jlong, jlong, jint, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetInputFocus
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetInputFocus
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSetInputFocus
 * Signature: (JJJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSetInputFocus
  (JNIEnv *, jclass, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSetWindowSize
 * Signature: (JJIIZ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSetWindowSize
  (JNIEnv *, jclass, jlong, jlong, jint, jint, jboolean);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetX
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetX
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetY
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetY
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetWidth
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetWidth
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetHeight
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetHeight
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nDestroyWindow
 * Signature: (JJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nDestroyWindow
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSwitchDisplayMode
 * Signature: (JIILorg/lwjgl/opengl/DisplayMode;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSwitchDisplayMode
  (JNIEnv *, jclass, jlong, jint, jint, jobject);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nInternAtom
 * Signature: (JLjava/lang/String;Z)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nInternAtom
  (JNIEnv *, jclass, jlong, jstring, jboolean);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetGammaRampLength
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetGammaRampLength
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSetGammaRamp
 * Signature: (JILjava/nio/ByteBuffer;)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSetGammaRamp
  (JNIEnv *, jclass, jlong, jint, jobject);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nConvertToNativeRamp
 * Signature: (Ljava/nio/FloatBuffer;II)Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nConvertToNativeRamp
  (JNIEnv *, jclass, jobject, jint, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetCurrentXRandrMode
 * Signature: (JI)Lorg/lwjgl/opengl/DisplayMode;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetCurrentXRandrMode
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSetTitle
 * Signature: (JJJI)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSetTitle
  (JNIEnv *, jclass, jlong, jlong, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSetClassHint
 * Signature: (JJJJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSetClassHint
  (JNIEnv *, jclass, jlong, jlong, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nReshape
 * Signature: (JJIIII)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nReshape
  (JNIEnv *, jclass, jlong, jlong, jint, jint, jint, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetAvailableDisplayModes
 * Signature: (JII)[Lorg/lwjgl/opengl/DisplayMode;
 */
JNIEXPORT jobjectArray JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetAvailableDisplayModes
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSync
 * Signature: (JZ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSync
  (JNIEnv *, jclass, jlong, jboolean);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nIconifyWindow
 * Signature: (JJI)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nIconifyWindow
  (JNIEnv *, jclass, jlong, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetNativeCursorCapabilities
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetNativeCursorCapabilities
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetMinCursorSize
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetMinCursorSize
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetMaxCursorSize
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetMaxCursorSize
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nCreateCursor
 * Signature: (JIIIIILjava/nio/IntBuffer;ILjava/nio/IntBuffer;I)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nCreateCursor
  (JNIEnv *, jclass, jlong, jint, jint, jint, jint, jint, jobject, jint, jobject, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nCreateBlankCursor
 * Signature: (JJ)J
 */
JNIEXPORT jlong JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nCreateBlankCursor
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nDestroyCursor
 * Signature: (JJ)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nDestroyCursor
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nGetPbufferCapabilities
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nGetPbufferCapabilities
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     org_lwjgl_opengl_LinuxDisplay
 * Method:    nSetWindowIcon
 * Signature: (JJLjava/nio/ByteBuffer;I)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_LinuxDisplay_nSetWindowIcon
  (JNIEnv *, jclass, jlong, jlong, jobject, jint);

#ifdef __cplusplus
}
#endif
#endif
