/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_lwjgl_opengl_MacOSXCanvasPeerInfo */

#ifndef _Included_org_lwjgl_opengl_MacOSXCanvasPeerInfo
#define _Included_org_lwjgl_opengl_MacOSXCanvasPeerInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_lwjgl_opengl_MacOSXCanvasPeerInfo
 * Method:    nInitHandle
 * Signature: (Ljava/nio/ByteBuffer;Ljava/nio/ByteBuffer;Ljava/nio/ByteBuffer;ZZII)Ljava/nio/ByteBuffer;
 */
JNIEXPORT jobject JNICALL Java_org_lwjgl_opengl_MacOSXCanvasPeerInfo_nInitHandle
  (JNIEnv *, jclass, jobject, jobject, jobject, jboolean, jboolean, jint, jint);

/*
 * Class:     org_lwjgl_opengl_MacOSXCanvasPeerInfo
 * Method:    nSetLayerPosition
 * Signature: (Ljava/nio/ByteBuffer;II)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXCanvasPeerInfo_nSetLayerPosition
  (JNIEnv *, jclass, jobject, jint, jint);

/*
 * Class:     org_lwjgl_opengl_MacOSXCanvasPeerInfo
 * Method:    nSetLayerBounds
 * Signature: (Ljava/nio/ByteBuffer;IIII)V
 */
JNIEXPORT void JNICALL Java_org_lwjgl_opengl_MacOSXCanvasPeerInfo_nSetLayerBounds
  (JNIEnv *, jclass, jobject, jint, jint, jint, jint);

#ifdef __cplusplus
}
#endif
#endif
